
/*
 This file is part of moth. moth is a program for creating
 and editing textured 3D models.
 
 Copyright (C) 2004 Peter Uray.

 moth is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 moth is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with moth; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#ifndef __MCOORDINATESYSTEM_H__
#define __MCOORDINATESYSTEM_H__

#include <moth/MVector.h>

class MCoordinateSystem
{
 public:
   
   static MCoordinateSystem *get(void);
   
   void render(void);
   float snapFunc(float x, float a);
 
   void setGridMode(int displayXY, int displayYZ, int displayZX);
   int getGridModeXY(void) const;
   int getGridModeYZ(void) const;
   int getGridModeZX(void) const;
   
   void setGridDistances(float x, float y, float z);
   float getGridDX(void) const;
   float getGridDY(void) const;
   float getGridDZ(void) const;
   
   void setGridPositions(float x, float y, float z);
   float getGridX(void) const;
   float getGridY(void) const;
   float getGridZ(void) const;
   
   void setSnap(float x, float y, float z);
   float getSnapX(void) const;
   float getSnapY(void) const;
   float getSnapZ(void) const;
   
   
   void setSnapAngles(float x, float y, float z);
   float getSnapAngleX(void) const;
   float getSnapAngleY(void) const;
   float getSnapAngleZ(void) const;
   
   
   void setSnap(bool mode);
   bool getSnapMode(void) const;
   void setSnapAngleMode(bool mode);
   bool getSnapAngleMode(void) const;
   
   void setRenderAxes(bool mode);
   bool getRenderAxesMode(void) const;
   
   void setArrows(bool mode);
   
   // this method snaps the vector coordinates to the closest grid point.
   void snap(MVector &x);
   
   // this method snaps the angle values contained in a to the snap angles.
   void snapAngles(MVector &a);
 
   // axes are rendered from -size to size since infinite axes require too much storage ;)
   void setSize(float size);
   float getSize(void) const;
 
   void setGridColor(float r, float g, float b);
   
   void readParameters(void);
 
 protected:

   MCoordinateSystem(void);
 
   float _gridX, _gridY, _gridZ;
   float _gridDX, _gridDY, _gridDZ;

   float _snapX, _snapY, _snapZ;
   float _snapAngleX, _snapAngleY, _snapAngleZ;
   
   float _gridR, _gridG, _gridB;
   
   float _size;
   
   bool _useSnap;
   bool _useAngleSnap;
   bool _renderAxes;
   
   int _displayGridXY;
   int _displayGridYZ;
   int _displayGridZX;
   
   bool _arrowMode;
   
   static MCoordinateSystem *_this;
};



#endif
