
/*
 This file is part of moth. moth is a program for creating
 and editing textured 3D models.
 
 Copyright (C) 2004 Peter Uray.

 moth is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 moth is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with moth; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#ifndef __MDOCUMENT_H__
#define __MDOCUMENT_H__

#include <qptrlist.h>

#include <moth/MModel.h>
#include <moth/MShape.h>
#include <moth/MLightSource.h>

class MDocument
{
 public:
 
   static MDocument *get(void);
   void clear(void);

   void setSelectedItem(MSceneGraphItem *item);
   MSceneGraphItem *getSelectedItem();
   
   void setMaster(void);
   void clearMaster(void);
   MShape *getMaster(void);
   
   MModel *getSelectedModel(void);
   MShape *getSelectedShape(void);
   MLightSource *getSelectedLightSource(void);
   
   void commit(void);
   void undo(void);
   void redo(void);
   
   bool canUndo(void);
   bool canRedo(void);
   
   bool loadScene(QString filename);
   
   
   void lock(void);
   void unlock(void);
 
 protected:

   MDocument(void);
   virtual ~MDocument(void);
  
 private:
 
   /// private stack operations.
   void push(QPtrList<QString> &list, QString *value);
   QString *pop(QPtrList<QString> &list);
 
   /// A pointer to the document singleton.
   static MDocument *_this;
   
   /// Buffers for storing special items..
   MSceneGraphItem *_selected;
   MShape *_master;
   
   /// The document history.
   QPtrList<QString> _past, _future;
   
   bool _lock;
};



#endif
