
/*
 This file is part of moth. moth is a program for creating
 and editing textured 3D models.
 
 Copyright (C) 2004 Peter Uray.

 moth is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 moth is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with moth; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#ifndef __MITEMMANIPULATOR_H__
#define __MITEMMANIPULATOR_H__

#include <qvaluevector.h>

#include <moth/MLine.h>
#include <MGLView.h>
#include <MDocument.h>
 
class MItemManipulator
{
 public:
 
   static MItemManipulator *get(void);
   
   void render(void);
   
   void set(MSceneGraphItem *item);
   
   MGLView::MAxis findAxis(MLine ray, float epsilon);


   void addVertex(int index);
   void removeVertex(int index);
   void clearVertexStack(void);
   unsigned int getVertexStackSize(void);
   int getVertexStackEntry(int index);
   bool hasVertexStackEntry(int index);

   void selectVertices(MLine &ray);
   int selectMasterVertex(MLine &ray);
   void setEraseVertexTo(bool value);
 
 protected:
 
   MItemManipulator(void);
 
   static MItemManipulator *_this;
 
   MSceneGraphItem *_item;
   
   MVector _x0, _x1, _x2, _x3;
   
   QValueVector<int> _vertexStack;
   
   bool _eraseVertex;
};


#endif
