
/*
 This file is part of moth. moth is a program for creating
 and editing textured 3D models.
 
 Copyright (C) 2004 Peter Uray.

 moth is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 moth is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with moth; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#ifndef __MTEXTUREVIEW_H__
#define __MTEXTUREVIEW_H__

#include <moth/MVector.h>

#include <MGLView.h>

class MTextureView : public MGLView
{
 Q_OBJECT
 
 signals: 
   
   // No signals.
   
 public slots:
 
   // No slots.

 public:
 
   MTextureView(QWidget *parent, const char *name);
   virtual ~MTextureView();

   int getNumTiles(void);
   void setNumTiles(int num);

   float getMaxLength(void);
   bool displaysSingleTexture(void);
   MVector getPosition(void);

 protected:
 
   virtual void renderView(void);
 
   virtual QString mousePosition3D(void);
 
   /// High level event handlers.
   virtual void onSelectModel(void) { }
   virtual void onSelectShape(void);
   virtual void onSelectLight(void) { }
  
   virtual void onMoveModelInit(void) { }
   virtual void onMoveModelPerform(void) { }
   virtual void onMoveModelFinish(void) { }

   virtual void onMoveShapeInit(void);
   virtual void onMoveShapePerform(void);
   virtual void onMoveShapeFinish(void);

   virtual void onMoveLightInit(void) { }
   virtual void onMoveLightPerform(void) { }
   virtual void onMoveLightFinish(void) { }
   

   virtual void onRotateModelInit(void) { }
   virtual void onRotateModelPerform(void) { }
   virtual void onRotateModelFinish(void) { }

   virtual void onRotateShapeInit(void);
   virtual void onRotateShapePerform(void);
   virtual void onRotateShapeFinish(void);

   virtual void onRotateLightInit(void) { }
   virtual void onRotateLightPerform(void) { }
   virtual void onRotateLightFinish(void) { }

   
   virtual void onScaleModelInit(void) { }
   virtual void onScaleModelPerform(void) { }
   virtual void onScaleModelFinish(void) { }

   virtual void onScaleShapeInit(void);
   virtual void onScaleShapePerform(void);
   virtual void onScaleShapeFinish(void);

   virtual void onScaleLightInit(void) { }
   virtual void onScaleLightPerform(void) { }
   virtual void onScaleLightFinish(void) { }
   
   
   virtual void onSelectVertexInit(void) { }
   virtual void onSelectVertexPerform(void) { }
   virtual void onSelectVertexFinish(void) { }

   
   virtual void onCreateVertex(void) { }
   virtual void onCreateTriangle(void) { }
   
   virtual void onMoveVertexInit(void);
   virtual void onMoveVertexPerform(void);
   virtual void onMoveVertexFinish(void);
   
   virtual void onDeleteVertex(void) { }
   virtual void onDeleteTriangle(void) { }
   virtual void onFlipEdge(void) { }
   
   virtual void onRotateView(void) { }
   virtual void onZoomView(void);
   virtual void onMoveView(void);
   virtual void onResetView(void);
   
   int _numTiles;
   
   float _minU, _minV, _maxU, _maxV;
   MVector _position;
   
   MVector *_selectedVertex;
   
   unsigned int *_background;
	 
	 MCamera _camera;
};


#endif
