
/*
 This file is part of moth. moth is a program for creating
 and editing textured 3D models.
 
 Copyright (C) 2004 Peter Uray.

 moth is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 moth is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with moth; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <qgl.h>

#include <MPreferences.h>
#include <MCoordinateSystem.h>

MCoordinateSystem *MCoordinateSystem::_this = NULL;


MCoordinateSystem *MCoordinateSystem::get(void)
{
 if(_this == NULL) _this = new MCoordinateSystem();
 return _this;
}


float MCoordinateSystem::snapFunc(float x, float a)
{
 float g1, g2;

 if(x >= 0.0)
 {
  g1 = a*(float)((int)(x/a));
  g2 = g1 + a;

  if(fabs(x - g1) < fabs(x - g2)) return g1;
  return g2;
 }
 else
 {
  x = -x;
  g1 = a*(float)((int)(x/a));
  g2 = g1 + a;

  if(fabs(x - g1) < fabs(x - g2)) return -g1;
  return -g2;
 }
}


MCoordinateSystem::MCoordinateSystem(void)
{
 readParameters();
}


void MCoordinateSystem::render(void)
{
 int i;
 int numGrid;
 
 float px, py, pz;

 // Disable Texuring, lighting  and blending in any case!
 glDisable(GL_TEXTURE_2D);
 glDisable(GL_LIGHTING);
 glDisable(GL_BLEND);
 
 // render the axes.
 if(_renderAxes == true)
 {
  glLineWidth(2.0); 
  // x-axis:
  px = _size + 0.5*_gridDX;
  py = 0.15*_gridDY;
  pz = 0.15*_gridDZ;
  glBegin(GL_LINES);
  glColor3f(1.0, 0.0, 0.0);
  glVertex3f(-_size - _gridDX, 0.0, 0.0); glVertex3f(_size + _gridDX, 0.0, 0.0);
  glEnd();
 
 
  if(_arrowMode == true)
  {
   glBegin(GL_TRIANGLES);
  
   glColor3f(1.0, 0.0, 0.0);
   glVertex3f(_size + 1.5*_gridDX, 0.0, 0.0); 
   glVertex3f(px, py, 0.0);
   glVertex3f(px, 0.0, pz);
   glColor3f(0.8, 0.0, 0.0);
   glVertex3f(_size + 1.5*_gridDX, 0.0, 0.0); 
   glVertex3f(px, 0.0, pz);
   glVertex3f(px, -py, 0.0);
 
   glColor3f(0.8, 0.0, 0.0);
   glVertex3f(_size + 1.5*_gridDX, 0.0, 0.0); 
   glVertex3f(px, 0.0, -pz);
   glVertex3f(px, py, 0.0);
   glColor3f(0.6, 0.0, 0.0);
   glVertex3f(_size + 1.5*_gridDX, 0.0, 0.0); 
   glVertex3f(px, -py, 0.0);
   glVertex3f(px, 0.0, -pz);
  
   glEnd();
  }
  
  // y-axis:
  px = 0.15*_gridDX;
  py = _size + 0.5*_gridDY;
  pz = 0.15*_gridDZ;
  glBegin(GL_LINES);
  glColor3f(0.0, 1.0, 0.0);
  glVertex3f(0.0, -_size - _gridDY, 0.0); glVertex3f(0.0, _size + _gridDY, 0.0);
  glEnd();
  
  
  if(_arrowMode == true)
  {
   glBegin(GL_TRIANGLES);
  
   glColor3f(0.0, 1.0, 0.0);
   glVertex3f(0.0, _size + 1.5*_gridDY, 0.0); 
   glVertex3f(-px, py, 0.0);
   glVertex3f(0.0, py, pz);
   glColor3f(0.0, 0.8, 0.0);
   glVertex3f(0.0, _size + 1.5*_gridDX, 0.0); 
   glVertex3f(0.0, py, pz);
   glVertex3f(px, py, 0.0);
   
   glColor3f(0.0, 0.8, 0.0);
   glVertex3f(0.0, _size + 1.5*_gridDY, 0.0); 
   glVertex3f(0.0, py, -pz);
   glVertex3f(-px, py, 0.0);
   glColor3f(0.0, 0.6, 0.0);
   glVertex3f(0.0, _size + 1.5*_gridDX, 0.0); 
   glVertex3f(px, py, 0.0);
   glVertex3f(0.0, py, -pz);
  
   glEnd();
  }
  
  // z-axis:
  px = 0.15*_gridDX;
  py = 0.15*_gridDY;
  pz = _size + 0.5*_gridDZ;
  
  glBegin(GL_LINES);
  glColor3f(0.0, 0.0, 1.0);
  glVertex3f(0.0, 0.0, -_size - _gridDZ); glVertex3f(0.0, 0.0, _size + _gridDZ);
  glEnd();
   
  if(_arrowMode == true)
  {
   glBegin(GL_TRIANGLES);
  
   glColor3f(0.0, 0.0, 1.0);
   glVertex3f(0.0, 0.0, _size + 1.5*_gridDZ); 
   glVertex3f(-px, 0.0, pz);
   glVertex3f(0.0, py, pz);
   glColor3f(0.0, 0.0, 0.8);
   glVertex3f(0.0, 0.0, _size + 1.5*_gridDZ); 
   glVertex3f(0.0, py, pz);
   glVertex3f(px, 0.0, pz);
  
   glColor3f(0.0, 0.0, 0.8);
   glVertex3f(0.0, 0.0, _size + 1.5*_gridDZ);
   glVertex3f(0.0, -py, pz); 
   glVertex3f(-px, 0.0, pz);
   glColor3f(0.0, 0.0, 0.6);
   glVertex3f(0.0, 0.0, _size + 1.5*_gridDZ); 
   glVertex3f(px, 0.0, pz);
   glVertex3f(0.0, -py, pz);
  
   glEnd();
  }
  glLineWidth(1.0); 
 }
 // render the grid
 if(_displayGridXY == 1)
 {
  glColor3f(_gridR, _gridG, _gridB);
  numGrid = (int)(_size / _gridDX);
  glBegin(GL_LINES);
  for(i = -numGrid; i <= numGrid; i++)
  {
   glVertex3f((float)i*_gridDX, -_size, _gridZ); glVertex3f((float)i*_gridDX, _size, _gridZ);
  }
  
  
  numGrid = (int)(_size / _gridDY);
  glBegin(GL_LINES);
  for(i = -numGrid; i <= numGrid; i++)
  {
   glVertex3f(-_size, (float)i*_gridDY, _gridZ); glVertex3f(_size, (float)i*_gridDY, _gridZ);
  }
  glEnd();
 }

 if(_displayGridYZ == 1)
 {
  glColor3f(_gridR, _gridG, _gridB);
  numGrid = (int)(_size / _gridDY);
  glBegin(GL_LINES);
  for(i = -numGrid; i <= numGrid; i++)
  {
   glVertex3f(_gridX, (float)i*_gridDY, -_size); glVertex3f(_gridX, (float)i*_gridDY, _size);
  }
  
  
  numGrid = (int)(_size / _gridDZ);
  glBegin(GL_LINES);
  for(i = -numGrid; i <= numGrid; i++)
  {
   glVertex3f(_gridX, -_size, (float)i*_gridDZ); glVertex3f(_gridX, _size, (float)i*_gridDZ);
  }
  glEnd();
 }
 
 if(_displayGridZX == 1)
 {
  glColor3f(_gridR, _gridG, _gridB);
  numGrid = (int)(_size / _gridDZ);
  glBegin(GL_LINES);
  for(i = -numGrid; i <= numGrid; i++)
  {
   glVertex3f(-_size, _gridY, (float)i*_gridDZ); glVertex3f(_size, _gridY, (float)i*_gridDZ);
  }
  
  
  numGrid = (int)(_size / _gridDX);
  glBegin(GL_LINES);
  for(i = -numGrid; i <= numGrid; i++)
  {
   glVertex3f((float)i*_gridDX, _gridY, -_size); glVertex3f((float)i*_gridDX, _gridY, _size);
  }
  glEnd();
 }
}


void MCoordinateSystem::setGridMode(int displayXY, int displayYZ, int displayZX)
{
 _displayGridXY = displayXY;
 _displayGridYZ = displayYZ;
 _displayGridZX = displayZX;
}


int MCoordinateSystem::getGridModeXY(void) const { return _displayGridXY; }
int MCoordinateSystem::getGridModeYZ(void) const { return _displayGridYZ; }
int MCoordinateSystem::getGridModeZX(void) const { return _displayGridZX; }


void MCoordinateSystem::setGridDistances(float x, float y, float z)
{
 _gridDX = x;
 _gridDY = y;
 _gridDZ = z;
}


float MCoordinateSystem::getGridDX(void) const { return _gridDX; }
float MCoordinateSystem::getGridDY(void) const { return _gridDY; }
float MCoordinateSystem::getGridDZ(void) const { return _gridDZ; }


void MCoordinateSystem::setGridPositions(float x, float y, float z)
{
 _gridX = x;
 _gridY = y;
 _gridZ = z;
}


float MCoordinateSystem::getGridX(void) const { return _gridX; }
float MCoordinateSystem::getGridY(void) const { return _gridY; }
float MCoordinateSystem::getGridZ(void) const { return _gridZ; }



void MCoordinateSystem::setSnap(float x, float y, float z)
{
 _snapX = x;
 _snapY = y;
 _snapZ = z;
}


float MCoordinateSystem::getSnapX(void) const { return _snapX; }
float MCoordinateSystem::getSnapY(void) const { return _snapY; }
float MCoordinateSystem::getSnapZ(void) const { return _snapZ; }



void MCoordinateSystem::setSnapAngles(float x, float y, float z)
{
 _snapAngleX = x;
 _snapAngleY = y;
 _snapAngleZ = z;
}


float MCoordinateSystem::getSnapAngleX(void) const { return _snapAngleX; }
float MCoordinateSystem::getSnapAngleY(void) const { return _snapAngleY; }
float MCoordinateSystem::getSnapAngleZ(void) const { return _snapAngleZ; }




void MCoordinateSystem::setSnap(bool mode)
{
 _useSnap = mode;
}

bool MCoordinateSystem::getSnapMode(void) const { return _useSnap; }


void MCoordinateSystem::setSnapAngleMode(bool mode)
{
 _useAngleSnap = mode;
}

bool MCoordinateSystem::getSnapAngleMode(void) const { return _useAngleSnap; }


void MCoordinateSystem::setArrows(bool mode)
{
 _arrowMode = mode;
}


void MCoordinateSystem::setRenderAxes(bool mode)
{
 _renderAxes = mode;
}


bool MCoordinateSystem::getRenderAxesMode(void) const
{
 return _renderAxes;
}


void MCoordinateSystem::snap(MVector &x)
{
 if(_useSnap == false) return;
 
 x[0] = snapFunc(x[0], _snapX);
 x[1] = snapFunc(x[1], _snapY);
 x[2] = snapFunc(x[2], _snapZ);
}


void MCoordinateSystem::snapAngles(MVector &a)
{
 if(_useAngleSnap == false) return;
 
 a[0] = snapFunc(a[0], _snapAngleX);
 a[1] = snapFunc(a[1], _snapAngleY);
 a[2] = snapFunc(a[2], _snapAngleZ);
}


void MCoordinateSystem::setSize(float size)
{
 _size = size;
}


float MCoordinateSystem::getSize(void) const { return _size; }


void MCoordinateSystem::setGridColor(float r, float g, float b)
{
 _gridR = r;
 _gridG = g;
 _gridB = b;
}


void MCoordinateSystem::readParameters(void)
{
 _gridX = MPreferences::get()->getGridX();
 _gridY = MPreferences::get()->getGridY();
 _gridZ = MPreferences::get()->getGridZ();
 
 _gridDX = MPreferences::get()->getGridDX();
 _gridDY = MPreferences::get()->getGridDY();
 _gridDZ = MPreferences::get()->getGridDZ();
 
 _snapX = MPreferences::get()->getSnapX();
 _snapY = MPreferences::get()->getSnapY();
 _snapZ = MPreferences::get()->getSnapZ();
 
 _snapAngleX = MPreferences::get()->getAngleSnapX();
 _snapAngleY = MPreferences::get()->getAngleSnapY();
 _snapAngleZ = MPreferences::get()->getAngleSnapZ();
 
 _gridR = 0.7;
 _gridG = 0.7;
 _gridB = 0.7;
   
 _size = MPreferences::get()->getSize();
   
 _useSnap = MPreferences::get()->getSnap();
 _useAngleSnap = MPreferences::get()->getSnapAngles();
 _renderAxes = MPreferences::get()->getDisplayAxes();
 
 _displayGridXY = MPreferences::get()->getDisplayXY();
 _displayGridYZ = MPreferences::get()->getDisplayYZ();
 _displayGridZX = MPreferences::get()->getDisplayZX();
 
 _arrowMode = true;
}
