
/*
 This file is part of moth. moth is a program for creating
 and editing textured 3D models.
 
 Copyright (C) 2004 Peter Uray.

 moth is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 moth is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with moth; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <iostream>

#include <qlabel.h>
#include <qlayout.h>
#include <qhbox.h>
#include <qvalidator.h>

#include <MCreatorDialog.h>

MCreatorDialog::MCreatorDialog(MCreator *creator) : QDialog(0, "creatorDialog", true, Qt::WStyle_DialogBorder | Qt::WStyle_Dialog | Qt::WStyle_StaysOnTop)
{
 QGridLayout* gl;
 unsigned int i;
 int ivalue;
 float fvalue;
 QString key;
 QVariant value;

 QLineEdit *editPtr;
 
 _creator = creator;

 setCaption("Parameters:");
 
 gl = new QGridLayout(this, _creator->numKeys() + 2, 4, 3); 
 
 
 for(i = 0; i < _creator->numKeys(); i++)
 {
  key = _creator->getKey(i);
  value = _creator->getParameter(key);
 
  switch(value.type())
  {
   case QVariant::Int:
    
     ivalue = value.toInt();
     
     gl->addMultiCellWidget(new QLabel(key, this), i, i, 0, 1);
 
     editPtr = new QLineEdit(this);
     editPtr->setAlignment(AlignRight);
     editPtr->setMinimumWidth(50);
     editPtr->setMaximumWidth(70);
     editPtr->setValidator(new QIntValidator(-1000000, 1000000, editPtr));
     
     editPtr->setText(value.toString());
     
     gl->addMultiCellWidget(editPtr, i, i, 2, 3);

     _numberEdit.append(editPtr);
     
   break;
   
   case QVariant::Double:
 
     fvalue = value.toDouble();
 
     gl->addMultiCellWidget(new QLabel(key, this), i, i, 0, 1);
 
     editPtr = new QLineEdit(this);
     editPtr->setAlignment(AlignRight);
     editPtr->setMinimumWidth(50);
     editPtr->setMaximumWidth(70);
     editPtr->setValidator(new QDoubleValidator(-9999999.0, 9999999.0, 2, editPtr));
     
     editPtr->setText(value.toString());
     
     gl->addMultiCellWidget(editPtr, i, i, 2, 3);

     _numberEdit.append(editPtr);
     
   break;
   
   default:
   
   break;
  }
 }

 gl->setRowStretch(_creator->numKeys(), 10);

  QHBox *hbox = new QHBox(this);
 _okButton = new QPushButton("OK", hbox);
 connect(_okButton, SIGNAL(released()), this, SLOT(onOKButtonPressed()));
 
 _cancelButton = new QPushButton("Cancel", hbox);
 connect(_cancelButton, SIGNAL(released()), this, SLOT(reject()));
 gl->addMultiCellWidget(hbox, _creator->numKeys() + 1, _creator->numKeys() + 1, 0, 3);
}


MCreatorDialog::~MCreatorDialog(void)
{

}


void MCreatorDialog::onOKButtonPressed(void)
{
 unsigned int i;
 QString key;
 QString edittext;

 
 for(i = 0; i < _creator->numKeys(); i++)
 {
  key = _creator->getKey(i);
 
  edittext = _numberEdit.at(i)->text();
  _creator->setParameter(key, QVariant(edittext));
 }
 
 accept();
}


void MCreatorDialog::show(void)
{
 QDialog::show();
}
