
/*
 This file is part of moth. moth is a program for creating
 and editing textured 3D models.
 
 Copyright (C) 2004 Peter Uray.

 moth is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 moth is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with moth; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <qvaluevector.h> 

#include <moth/MShape.h>

#include <MDocument.h>
#include <MElevationCreator.h>


int MElevationCreator::vertexAt(int phi, int theta, int sectors, int rings)
{
 if(theta < 0) return 0;
 else if(phi < sectors) return theta*sectors + phi;
 
 else return theta*sectors;
}


int MElevationCreator::texVertexAt(int phi, int theta, int sectors, int rings)
{
 return theta*(sectors + 1) + phi;
}


MElevationCreator::MElevationCreator(void) : MCreator()
{

}


MShape *MElevationCreator::create(MModel *parent)
{
 int i, j;

 int rings, sectors;
 float height;
 float sizeU, sizeV;
 MVector dz;
 MVertex *v;
 QValueVector<MVector> vertices;
 
 float x, y, z; 
 MShape *shape;
 
 rings = _parameters[QString("Rings")].toInt();
 height = _parameters[QString("Height")].toDouble();
 
 sizeU = _parameters[QString("U-Size")].toDouble();
 sizeV = _parameters[QString("V-Size")].toDouble();
 
 shape = MDocument::get()->getSelectedShape();
 if(shape)
 {
  shape->setName(shape->getName() + " elevation");
  
  // vertices
  QPtrList<MVertex> vl = shape->getVertexList();
  for(v = vl.first(); v; v = vl.next())
  {
   vertices.push_back(v->getPosition());
  }
  sectors = (int)vertices.size();
  
  for(j = 1; j <= rings; j++)
  {
   dz = j*(height / ((float)(rings - 1))) * shape->getUp();
 
   for(i = 0; i < sectors; i++)
   {
     x = vertices[i][0] + dz[0];
     y = vertices[i][1] + dz[1];
     z = vertices[i][2] + dz[2];
  
     shape->addVertex(new MVertex(x, y, z));
   }
  }
 
   // textureVertices
  for(j = 0; j <= rings; j++)
  {
   for(i = 0; i <= sectors; i++)
   {
    x = sizeU*((float)i)/((float) sectors);
    y = sizeU*((float)j)/((float) rings);

    shape->addTextureVertex(new MVector(x, y, 0.0));
   }
  }
 
  // facets
  for(j = 0; j < rings; j++)
  {
   for(i = 0; i < sectors; i++)
   {
    shape->addFacet(new MFacet(vertexAt(i+1, j, sectors, rings),
                               vertexAt(i, j, sectors, rings),
                               vertexAt(i, j+1, sectors, rings),
			      
	 		       texVertexAt(i+1, j, sectors, rings),
                               texVertexAt(i, j, sectors, rings),
                               texVertexAt(i, j+1, sectors, rings)));
			      
			      
			      
    shape->addFacet(new MFacet(vertexAt(i+1, j, sectors, rings),
                               vertexAt(i, j+1, sectors, rings),
                               vertexAt(i+1, j+1, sectors, rings),
		 	      
	 		       texVertexAt(i+1, j, sectors, rings),
                               texVertexAt(i, j+1, sectors, rings),
                               texVertexAt(i+1, j+1, sectors, rings)));
   }
  }
 }
 
 return shape;
}
