
/*
 This file is part of moth. moth is a program for creating
 and editing textured 3D models.
 
 Copyright (C) 2004 Peter Uray.

 moth is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 moth is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with moth; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <qlabel.h>
#include <qlayout.h>
#include <qhbox.h>
#include <qstring.h>
#include <qfiledialog.h>
#include <qvalidator.h>

#include <MModelView.h>
#include <MGridAndUnitsWidget.h>


MGridAndUnitsWidget::MGridAndUnitsWidget(QWidget *parent) :
   QGroupBox(0, Horizontal, "", parent, "gridProps")
{
 QLabel *label;
 QHBox *hbox;
 
 setMargin(10);

 QGridLayout* gl = new QGridLayout(layout(), 11, 4);
 
 // grid position
 label = new QLabel("Grid Position:", this);
 gl->addWidget(label, 0, 0);
    
 _gridXEdit = new QLineEdit(this);
 _gridXEdit->setAlignment(AlignRight);
 _gridXEdit->setMinimumWidth(50);
 _gridXEdit->setMaximumWidth(50);
 _gridXEdit->setValidator(new QDoubleValidator(-9999999.0, 9999999.0, 2, _gridXEdit));
 gl->addWidget(_gridXEdit, 0, 1);
 
 _gridYEdit = new QLineEdit(this);
 _gridYEdit->setAlignment(AlignRight);
 _gridYEdit->setMinimumWidth(50);
 _gridYEdit->setMaximumWidth(50);
 _gridYEdit->setValidator(new QDoubleValidator(-9999999.0, 9999999.0, 2, _gridYEdit));
 gl->addWidget(_gridYEdit, 0, 2);

 _gridZEdit = new QLineEdit(this);
 _gridZEdit->setAlignment(AlignRight);
 _gridZEdit->setMinimumWidth(50);
 _gridZEdit->setMaximumWidth(50);
 _gridZEdit->setValidator(new QDoubleValidator(-9999999.0, 9999999.0, 2, _gridZEdit));
 gl->addWidget(_gridZEdit, 0, 3);



 // grid spacing
 label = new QLabel("Grid Spacing:", this);
 gl->addWidget(label, 1, 0);
    
 _gridDXEdit = new QLineEdit(this);
 _gridDXEdit->setAlignment(AlignRight);
 _gridDXEdit->setMinimumWidth(50);
 _gridDXEdit->setMaximumWidth(50);
 _gridDXEdit->setValidator(new QDoubleValidator(0.0, 9999999.0, 2, _gridDXEdit));
 gl->addWidget(_gridDXEdit, 1, 1);
 
 _gridDYEdit = new QLineEdit(this);
 _gridDYEdit->setAlignment(AlignRight);
 _gridDYEdit->setMinimumWidth(50);
 _gridDYEdit->setMaximumWidth(50);
 _gridDYEdit->setValidator(new QDoubleValidator(0.0, 9999999.0, 2, _gridDYEdit));
 gl->addWidget(_gridDYEdit, 1, 2);

 _gridDZEdit = new QLineEdit(this);
 _gridDZEdit->setAlignment(AlignRight);
 _gridDZEdit->setMinimumWidth(50);
 _gridDZEdit->setMaximumWidth(50);
 _gridDZEdit->setValidator(new QDoubleValidator(0.0, 9999999.0, 2, _gridDZEdit));
 gl->addWidget(_gridDZEdit, 1, 3);


 //snap spacing
 label = new QLabel("Snap Spacing:", this);
 gl->addWidget(label, 2, 0);
    
 _snapXEdit = new QLineEdit(this);
 _snapXEdit->setAlignment(AlignRight);
 _snapXEdit->setMinimumWidth(50);
 _snapXEdit->setMaximumWidth(50);
 _snapXEdit->setValidator(new QDoubleValidator(-9999999.0, 9999999.0, 2, _snapXEdit));
 gl->addWidget(_snapXEdit, 2, 1);
 
 _snapYEdit = new QLineEdit(this);
 _snapYEdit->setAlignment(AlignRight);
 _snapYEdit->setMinimumWidth(50);
 _snapYEdit->setMaximumWidth(50);
 _snapYEdit->setValidator(new QDoubleValidator(-9999999.0, 9999999.0, 2, _snapYEdit));
 gl->addWidget(_snapYEdit, 2, 2);

 _snapZEdit = new QLineEdit(this);
 _snapZEdit->setAlignment(AlignRight);
 _snapZEdit->setMinimumWidth(50);
 _snapZEdit->setMaximumWidth(50);
 _snapZEdit->setValidator(new QDoubleValidator(-9999999.0, 9999999.0, 2, _snapZEdit));
 gl->addWidget(_snapZEdit, 2, 3);


 // angle snap spacing
 label = new QLabel("Angular Spacing:", this);
 gl->addWidget(label, 3, 0);
    
 _snapAngleXEdit = new QLineEdit(this);
 _snapAngleXEdit->setAlignment(AlignRight);
 _snapAngleXEdit->setMinimumWidth(50);
 _snapAngleXEdit->setMaximumWidth(50);
 _snapAngleXEdit->setValidator(new QDoubleValidator(-9999999.0, 9999999.0, 2, _snapXEdit));
 gl->addWidget(_snapAngleXEdit, 3, 1);
 
 _snapAngleYEdit = new QLineEdit(this);
 _snapAngleYEdit->setAlignment(AlignRight);
 _snapAngleYEdit->setMinimumWidth(50);
 _snapAngleYEdit->setMaximumWidth(50);
 _snapAngleYEdit->setValidator(new QDoubleValidator(-9999999.0, 9999999.0, 2, _snapYEdit));
 gl->addWidget(_snapAngleYEdit, 3, 2);

 _snapAngleZEdit = new QLineEdit(this);
 _snapAngleZEdit->setAlignment(AlignRight);
 _snapAngleZEdit->setMinimumWidth(50);
 _snapAngleZEdit->setMaximumWidth(50);
 _snapAngleZEdit->setValidator(new QDoubleValidator(-9999999.0, 9999999.0, 2, _snapZEdit));
 gl->addWidget(_snapAngleZEdit, 3, 3);
 
 
 // Grid display switches
 _displayXYBox = new QCheckBox("XY Grid", this);
 gl->addMultiCellWidget(_displayXYBox, 4, 4, 0, 1);
 
 _displayYZBox = new QCheckBox("YZ Grid", this);
 gl->addMultiCellWidget(_displayYZBox, 4, 4, 2, 3);

 _displayZXBox = new QCheckBox("ZX Grid", this);
 gl->addMultiCellWidget(_displayZXBox, 5, 5, 0, 1);

 _displayAxesBox = new QCheckBox("Display Axes", this);
 gl->addMultiCellWidget(_displayAxesBox, 5, 5, 2, 3);
 
 _snapBox = new QCheckBox("Snap", this);
 gl->addMultiCellWidget(_snapBox, 6, 6, 0, 1);

 _snapAnglesBox = new QCheckBox("Snap Angles", this);
 gl->addMultiCellWidget(_snapAnglesBox, 6, 6, 2, 3);


 // grid size
 label = new QLabel("Grid Size:", this);
 gl->addMultiCellWidget(label, 7, 7, 0, 1);
    
 _sizeEdit = new QLineEdit(this);
 _sizeEdit->setAlignment(AlignRight);
 _sizeEdit->setMinimumWidth(40);
 _sizeEdit->setMaximumWidth(150);
 _sizeEdit->setValidator(new QDoubleValidator(1.0, 9999999.0, 2, _sizeEdit));
 gl->addMultiCellWidget(_sizeEdit, 7, 7, 2, 3);
 
 
 // texture grid size
 label = new QLabel("Texture Grid Size:", this);
 gl->addMultiCellWidget(label, 8, 8, 0, 1);
    
 _texSizeEdit = new QLineEdit(this);
 _texSizeEdit->setAlignment(AlignRight);
 _texSizeEdit->setMinimumWidth(40);
 _texSizeEdit->setMaximumWidth(150);
 _texSizeEdit->setValidator(new QIntValidator(1, 1000, _texSizeEdit));
 gl->addMultiCellWidget(_texSizeEdit, 8, 8, 2, 3);

 
 hbox = new QHBox(this);
 
 _applyButton = new QPushButton("Apply", hbox);
  connect(_applyButton, SIGNAL(released()), this, SLOT(onApply()));
 
 _resetButton = new QPushButton("Reset", hbox);
  connect(_resetButton, SIGNAL(released()), this, SLOT(onReset()));


 gl->addMultiCellWidget(hbox, 10, 10, 0, 3); 

 gl->setRowStretch(9, 10);
 _gridXEdit->setFocus();
}



void MGridAndUnitsWidget::onApply(void)
{
 int i, j, k;
 float x, y, z;

 sscanf(_gridXEdit->text(), "%f", &x);
 sscanf(_gridYEdit->text(), "%f", &y);
 sscanf(_gridZEdit->text(), "%f", &z);
 MCoordinateSystem::get()->setGridPositions(x, y, z);
 
 sscanf(_gridDXEdit->text(), "%f", &x);
 sscanf(_gridDYEdit->text(), "%f", &y);
 sscanf(_gridDZEdit->text(), "%f", &z);
 if(x < 0.0) x = 0.0; if(y < 0.0) y = 0.0; if(z < 0.0) z = 0.0; 
 MCoordinateSystem::get()->setGridDistances(x, y, z);
 
 sscanf(_snapXEdit->text(), "%f", &x);
 sscanf(_snapYEdit->text(), "%f", &y);
 sscanf(_snapZEdit->text(), "%f", &z);
 if(x < 0.0) x = 0.0; if(y < 0.0) y = 0.0; if(z < 0.0) z = 0.0; 
 MCoordinateSystem::get()->setSnap(x, y, z);
 
 sscanf(_snapAngleXEdit->text(), "%f", &x);
 sscanf(_snapAngleYEdit->text(), "%f", &y);
 sscanf(_snapAngleZEdit->text(), "%f", &z);
 if(x < 0.0) x = 0.0; if(y < 0.0) y = 0.0; if(z < 0.0) z = 0.0; 
 MCoordinateSystem::get()->setSnapAngles(x, y, z);
 
 i = 0; j = 0; k = 0;
 if(_displayXYBox->isChecked()) i = 1;
 if(_displayYZBox->isChecked()) j = 1;
 if(_displayZXBox->isChecked()) k = 1;
 MCoordinateSystem::get()->setGridMode(i, j, k);
 
 MCoordinateSystem::get()->setSnap(false);
 if(_snapBox->isChecked()) MCoordinateSystem::get()->setSnap(true);
 
 MCoordinateSystem::get()->setSnapAngleMode(false);
 if(_snapAnglesBox->isChecked()) MCoordinateSystem::get()->setSnapAngleMode(true);
 
 MCoordinateSystem::get()->setRenderAxes(false);
 if(_displayAxesBox->isChecked()) MCoordinateSystem::get()->setRenderAxes(true);
 
 sscanf(_sizeEdit->text(), "%f", &x);
 if(x < 0.0) x = 0.0;
 MCoordinateSystem::get()->setSize(x);
 
 sscanf(_texSizeEdit->text(), "%i", &i);
 MModelView::get()->getTextureView().setNumTiles(i);
 
 fetchData();
 
 MModelView::get()->repaint();
}


void MGridAndUnitsWidget::onReset(void)
{
 MCoordinateSystem::get()->resetToDefault();
 fetchData();
}


void MGridAndUnitsWidget::fetchData(void)
{
 char buffer[20];
 
 sprintf(buffer, "%8.3f", MCoordinateSystem::get()->getGridX());
 _gridXEdit->setText(buffer);
 sprintf(buffer, "%8.3f", MCoordinateSystem::get()->getGridY());
 _gridYEdit->setText(buffer);
 sprintf(buffer, "%8.3f", MCoordinateSystem::get()->getGridZ());
 _gridZEdit->setText(buffer);
 
 
 sprintf(buffer, "%8.3f", MCoordinateSystem::get()->getGridDX());
 _gridDXEdit->setText(buffer);
 sprintf(buffer, "%8.3f", MCoordinateSystem::get()->getGridDY());
 _gridDYEdit->setText(buffer);
 sprintf(buffer, "%8.3f", MCoordinateSystem::get()->getGridDZ());
 _gridDZEdit->setText(buffer);
 
 
 sprintf(buffer, "%8.3f", MCoordinateSystem::get()->getSnapX());
 _snapXEdit->setText(buffer);
 sprintf(buffer, "%8.3f", MCoordinateSystem::get()->getSnapY());
 _snapYEdit->setText(buffer);
 sprintf(buffer, "%8.3f", MCoordinateSystem::get()->getSnapZ());
 _snapZEdit->setText(buffer);
 
 sprintf(buffer, "%8.3f", MCoordinateSystem::get()->getSnapAngleX());
 _snapAngleXEdit->setText(buffer);
 sprintf(buffer, "%8.3f", MCoordinateSystem::get()->getSnapAngleY());
 _snapAngleYEdit->setText(buffer);
 sprintf(buffer, "%8.3f", MCoordinateSystem::get()->getSnapAngleZ());
 _snapAngleZEdit->setText(buffer);
 
 
 
 (MCoordinateSystem::get()->getGridModeXY() == 1) ? _displayXYBox->setChecked(true) : _displayXYBox->setChecked(false);
 (MCoordinateSystem::get()->getGridModeYZ() == 1) ? _displayYZBox->setChecked(true) : _displayYZBox->setChecked(false);
 (MCoordinateSystem::get()->getGridModeZX() == 1) ? _displayZXBox->setChecked(true) : _displayZXBox->setChecked(false);
 (MCoordinateSystem::get()->getSnapMode() == true) ? _snapBox->setChecked(true) : _snapBox->setChecked(false);
 (MCoordinateSystem::get()->getSnapAngleMode() == true) ? _snapAnglesBox->setChecked(true) : _snapAnglesBox->setChecked(false);
 (MCoordinateSystem::get()->getRenderAxesMode() == true) ? _displayAxesBox->setChecked(true) : _displayAxesBox->setChecked(false);


 sprintf(buffer, "%8.3f", MCoordinateSystem::get()->getSize());
 _sizeEdit->setText(buffer);
 
 sprintf(buffer, "%i", MModelView::get()->getTextureView().getNumTiles());
 _texSizeEdit->setText(buffer);
}



void MGridAndUnitsWidget::showEvent(QShowEvent *)
{
 fetchData();
}
