
/*
 This file is part of moth. moth is a program for creating
 and editing textured 3D models.
 
 Copyright (C) 2004 Peter Uray.

 moth is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 moth is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with moth; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <iostream>

#include <qlayout.h>
#include <qhbox.h>
#include <qlabel.h>
#include <qvalidator.h>
#include <qvariant.h>

#include <MModelView.h>
#include <MLightSourceParameterDialog.h>


MLightSourceParameterDialog::MLightSourceParameterDialog(MLightSource *lightSource)  : QDialog(0, "lightSourceParameterDialog", true, Qt::WStyle_DialogBorder | Qt::WStyle_Dialog | Qt::WStyle_StaysOnTop)
{
 QGridLayout* gl;
 QVariant value;
 float *params;
 int r, g, b;
 
 setCaption("Light Source Parameters");
 
 resize(450, 180);

 _lightSource = lightSource;
 
 gl = new QGridLayout(this, 8, 14, 3);
 
 
 /// ambient color
 gl->addMultiCellWidget(new QLabel("Ambient", this), 0, 0, 0, 2);
 
 _ambientDisplay = new QButton(this);
 
 params = _lightSource->getAmbient();
 r = (int)(255.0*params[0]);
 g = (int)(255.0*params[1]);
 b = (int)(255.0*params[2]);
 _ambientDisplay->setPaletteBackgroundColor(QColor(r, g, b));
 gl->addMultiCellWidget(_ambientDisplay, 1, 1, 0, 2);
 
 _ambientRSlider = new QSlider(Qt::Vertical, this);
 _ambientRSlider->setMinValue(0);
 _ambientRSlider->setMaxValue(255);
 _ambientRSlider->setValue(255 - r);
 connect(_ambientRSlider, SIGNAL(valueChanged(int)), this, SLOT(onSliderValueChanged(int)));
 gl->addMultiCellWidget(_ambientRSlider, 2, 4, 0, 0);
 
 _ambientGSlider = new QSlider(Qt::Vertical, this);
 _ambientGSlider->setMinValue(0);
 _ambientGSlider->setMaxValue(255);
 _ambientGSlider->setValue(255 - g);
 connect(_ambientGSlider, SIGNAL(valueChanged(int)), this, SLOT(onSliderValueChanged(int)));
 gl->addMultiCellWidget(_ambientGSlider, 2, 4, 1, 1);
 
 _ambientBSlider = new QSlider(Qt::Vertical, this);
 _ambientBSlider->setMinValue(0);
 _ambientBSlider->setMaxValue(255);
 _ambientBSlider->setValue(255 - b);
 connect(_ambientBSlider, SIGNAL(valueChanged(int)), this, SLOT(onSliderValueChanged(int)));
 gl->addMultiCellWidget(_ambientBSlider, 2, 4, 2, 2);
 

 
 /// diffuse color
 gl->addMultiCellWidget(new QLabel("Diffuse", this), 0, 0, 4, 6);
 
 _diffuseDisplay = new QButton(this);
 
 params = _lightSource->getDiffuse();
 r = (int)(255.0*params[0]);
 g = (int)(255.0*params[1]);
 b = (int)(255.0*params[2]);
 
 _diffuseDisplay->setPaletteBackgroundColor(QColor(r, g, b));
 gl->addMultiCellWidget(_diffuseDisplay, 1, 1, 4, 6);
 
 _diffuseRSlider = new QSlider(Qt::Vertical, this);
 _diffuseRSlider->setMinValue(0);
 _diffuseRSlider->setMaxValue(255);
 _diffuseRSlider->setValue(255 - r);
 connect(_diffuseRSlider, SIGNAL(valueChanged(int)), this, SLOT(onSliderValueChanged(int)));
 gl->addMultiCellWidget(_diffuseRSlider, 2, 4, 4, 4);
 
 _diffuseGSlider = new QSlider(Qt::Vertical, this);
 _diffuseGSlider->setMinValue(0);
 _diffuseGSlider->setMaxValue(255);
 _diffuseGSlider->setValue(255 - g);
 connect(_diffuseGSlider, SIGNAL(valueChanged(int)), this, SLOT(onSliderValueChanged(int)));
 gl->addMultiCellWidget(_diffuseGSlider, 2, 4, 5, 5);
 
 _diffuseBSlider = new QSlider(Qt::Vertical, this);
 _diffuseBSlider->setMinValue(0);
 _diffuseBSlider->setMaxValue(255);
 _diffuseBSlider->setValue(255 - b);
 connect(_diffuseBSlider, SIGNAL(valueChanged(int)), this, SLOT(onSliderValueChanged(int)));
 gl->addMultiCellWidget(_diffuseBSlider, 2, 4, 6, 6);

 
 
 
 
 
 /// specular color
 gl->addMultiCellWidget(new QLabel("Specular", this), 0, 0, 8, 10);
 
 _specularDisplay = new QButton(this);
 
 params = _lightSource->getSpecular();
 r = (int)(255.0*params[0]);
 g = (int)(255.0*params[1]);
 b = (int)(255.0*params[2]);
 
 _specularDisplay->setPaletteBackgroundColor(QColor(r, g, b));
 gl->addMultiCellWidget(_specularDisplay, 1, 1, 8, 10);
  
 _specularRSlider = new QSlider(Qt::Vertical, this);
 _specularRSlider->setMinValue(0);
 _specularRSlider->setMaxValue(255);
 _specularRSlider->setValue(255 - r);
 connect(_specularRSlider, SIGNAL(valueChanged(int)), this, SLOT(onSliderValueChanged(int)));
 gl->addMultiCellWidget(_specularRSlider, 2, 4, 8, 8);
 
 _specularGSlider = new QSlider(Qt::Vertical, this);
 _specularGSlider->setMinValue(0);
 _specularGSlider->setMaxValue(255);
 _specularGSlider->setValue(255 - g);
 connect(_specularGSlider, SIGNAL(valueChanged(int)), this, SLOT(onSliderValueChanged(int)));
 gl->addMultiCellWidget(_specularGSlider, 2, 4, 9, 9);
 
 _specularBSlider = new QSlider(Qt::Vertical, this);
 _specularBSlider->setMinValue(0);
 _specularBSlider->setMaxValue(255);
 _specularBSlider->setValue(255 - b);
 connect(_specularBSlider, SIGNAL(valueChanged(int)), this, SLOT(onSliderValueChanged(int)));
 gl->addMultiCellWidget(_specularBSlider, 2, 4, 10, 10);
 
 
 
 
 
 /// exponent, cutoff and attenuations.
 gl->addMultiCellWidget(new QLabel("Exponent:", this), 0, 0, 12, 12);
 
 value = _lightSource->getSpotExponent();
 _exponentEdit = new QLineEdit(value.asString(), this);
 
 _exponentEdit->setAlignment(AlignRight);
 _exponentEdit->setValidator(new QDoubleValidator(0.0, 1000.0, 2, _exponentEdit));
 connect(_exponentEdit, SIGNAL(textChanged(const QString &)), this, SLOT(onTextChanged(const QString &)));
 gl->addWidget(_exponentEdit, 0, 13);
 
 
 gl->addMultiCellWidget(new QLabel("Cutoff:", this), 1, 1, 12, 12);
 
 value = _lightSource->getSpotCutoff();
 _cutoffEdit = new QLineEdit(value.asString(), this);
 
 _cutoffEdit->setAlignment(AlignRight);
 _cutoffEdit->setValidator(new QDoubleValidator(0.0, 9999999.0, 2, _cutoffEdit));
 connect(_cutoffEdit, SIGNAL(textChanged(const QString &)), this, SLOT(onTextChanged(const QString &)));
 gl->addWidget(_cutoffEdit, 1, 13);
 
 
 gl->addMultiCellWidget(new QLabel("Constant Attenuation:", this), 2, 2, 12, 12);
 
 value = _lightSource->getConstantAttenuation();
 _constantAttenuationEdit = new QLineEdit(value.asString(), this);
 
 _constantAttenuationEdit->setAlignment(AlignRight);
 _constantAttenuationEdit->setValidator(new QDoubleValidator(0.0, 9999999.0, 2, _constantAttenuationEdit));
 connect(_constantAttenuationEdit, SIGNAL(textChanged(const QString &)), this, SLOT(onTextChanged(const QString &)));
 gl->addWidget(_constantAttenuationEdit, 2, 13);
 
 
 gl->addMultiCellWidget(new QLabel("Linear Attenuation:", this), 3, 3, 12, 12);

 value = _lightSource->getLinearAttenuation();
 _linearAttenuationEdit = new QLineEdit(value.asString(), this);

 _linearAttenuationEdit->setAlignment(AlignRight);
 _linearAttenuationEdit->setValidator(new QDoubleValidator(0.0, 9999999.0, 2, _linearAttenuationEdit));
 connect(_linearAttenuationEdit, SIGNAL(textChanged(const QString &)), this, SLOT(onTextChanged(const QString &)));
 gl->addWidget(_linearAttenuationEdit, 3, 13);
 
 
 gl->addMultiCellWidget(new QLabel("Quadratic Attenuation:", this), 4, 4, 12, 12);
 
 value = _lightSource->getQuadraticAttenuation();
 _quadraticAttenuationEdit = new QLineEdit(value.asString(), this);
 
 _quadraticAttenuationEdit->setAlignment(AlignRight);
 _quadraticAttenuationEdit->setValidator(new QDoubleValidator(0.0, 9999999.0, 2, _quadraticAttenuationEdit));
 connect(_quadraticAttenuationEdit, SIGNAL(textChanged(const QString &)), this, SLOT(onTextChanged(const QString &)));
 gl->addWidget(_quadraticAttenuationEdit, 4, 13);
 
 
 gl->setColStretch(3, 10); 
 gl->setColStretch(7, 10); 
 gl->setColStretch(11, 10);
 gl->setRowStretch(6, 10);
  
 QHBox *hbox = new QHBox(this);
 _okButton = new QPushButton("Dismiss", hbox);
 connect(_okButton, SIGNAL(released()), this, SLOT(onOKButtonPressed()));
 
 gl->addMultiCellWidget(hbox, 7, 7, 0, 13);
}


MLightSourceParameterDialog::~MLightSourceParameterDialog(void)
{

}


void MLightSourceParameterDialog::show(void)
{
 QDialog::show();
}


void MLightSourceParameterDialog::onOKButtonPressed(void)
{
 accept();
}


void MLightSourceParameterDialog::onSliderValueChanged(int value)
{
 int r, g, b;

 r = 255 - _ambientRSlider->value();
 g = 255 - _ambientGSlider->value();
 b = 255 - _ambientBSlider->value();
 _ambientDisplay->setPaletteBackgroundColor(QColor(r, g, b));
 _lightSource->setAmbient(r/255.0, g/255.0, b/255.0, 1.0);
 
 r = 255 - _diffuseRSlider->value();
 g = 255 - _diffuseGSlider->value();
 b = 255 - _diffuseBSlider->value();
 _diffuseDisplay->setPaletteBackgroundColor(QColor(r, g, b));
 _lightSource->setDiffuse(r/255.0, g/255.0, b/255.0, 1.0);
 
 r = 255 - _specularRSlider->value();
 g = 255 - _specularGSlider->value();
 b = 255 - _specularBSlider->value();
 _specularDisplay->setPaletteBackgroundColor(QColor(r, g, b));
 _lightSource->setSpecular(r/255.0, g/255.0, b/255.0, 1.0);

 MModelView::get()->repaint();
}



void MLightSourceParameterDialog::onTextChanged(const QString &text)
{
 QVariant exp, cut, con, lin, quad;

 exp = _exponentEdit->text();
 cut = _cutoffEdit->text();
 _lightSource->setSpotParams(exp.asDouble(), cut.asDouble());
 
 con = _constantAttenuationEdit->text();
 lin = _linearAttenuationEdit->text();
 quad = _quadraticAttenuationEdit->text();
 
 _lightSource->setAttenuationParams(con.asDouble(), lin.asDouble(), quad.asDouble());

 MModelView::get()->repaint();
}
