
/*
 This file is part of moth. moth is a program for creating
 and editing textured 3D models.
 
 Copyright (C) 2004 Peter Uray.

 moth is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 moth is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with moth; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <iostream>

#include <qtoolbutton.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qsplitter.h>
#include <qlistview.h>

#include <qtabwidget.h>
#include <qvbox.h>

#include <qheader.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qstatusbar.h>
#include <qapplication.h>

#include <qfile.h>
#include <qfiledialog.h>
#include <qtextstream.h>

#include <moth/MItemReader.h>

#include <MTooltips.h>

#include <MMainWindow.h>
#include <MTreeItem.h>

#include <MPreferences.h>
#include <MPreferencesDialog.h>

#include <MModelView.h>
#include <MStructureView.h>

#include <MItemManipulator.h>

#include <MDocument.h>

#include "icons/iconnew.xpm"
#include "icons/iconopen.xpm"
#include "icons/iconsave.xpm"
#include "icons/iconundo.xpm"
#include "icons/iconredo.xpm"

#include "icons/icon3d.xpm" 
#include "icons/iconxy.xpm"
#include "icons/iconyz.xpm"
#include "icons/iconzx.xpm"
#include "icons/iconquad.xpm"
#include "icons/icontex.xpm"

#include "icons/iconsel.xpm"
#include "icons/iconzoom.xpm"

#include "icons/iconmove.xpm"
#include "icons/iconrot.xpm"
#include "icons/iconscale.xpm"
#include "icons/iconmagnet.xpm"

#include "icons/iconselvertex.xpm"
#include "icons/iconaddvertex.xpm"
#include "icons/iconaddtriangle.xpm"

#include "icons/iconmovevertex.xpm"
#include "icons/iconrotatevertex.xpm"
#include "icons/iconscalevertex.xpm"

#include "icons/iconflipedge.xpm"
#include "icons/iconsnapvertex2master.xpm"

MMainWindow *MMainWindow::_this = NULL;

MMainWindow::MMainWindow(const char *caption) : QMainWindow(NULL, caption)
{
 QSplitter *centralSplitter;

 setCaption(caption);
 
 _fileMenu = new QPopupMenu(this);
 _fileMenu->insertItem("New Project ..", this, SLOT(onProjectNew()));
 _fileMenu->insertSeparator();
 _fileMenu->insertItem("&New Document", this, SLOT(onDocumentNew()), CTRL+Key_N);
 _fileMenu->insertItem("&Open Document", this, SLOT(onDocumentOpen()), CTRL+Key_O);
 _fileMenu->insertItem("&Save Document", this, SLOT(onDocumentSave()), CTRL+Key_S);
 _fileMenu->insertItem("Save Document as ..", this, SLOT(onDocumentSaveAs()));
 _fileMenu->insertSeparator();
 _fileMenu->insertItem("&Preferences ..", this, SLOT(onPreferences()), CTRL+Key_P);
 _fileMenu->insertSeparator();
 _fileMenu->insertItem("About", this, SLOT(onAbout()));
 _fileMenu->insertItem("&Quit", this, SLOT(onQuit()), CTRL+Key_Q);

 menuBar()->insertItem( "&File", _fileMenu);
 
 
 _editMenu = new QPopupMenu(this);
 _editMenu->insertItem("Undo", this, SLOT(onUndo()), CTRL+Key_Z);
 _editMenu->insertItem("Redo", this, SLOT(onRedo()), CTRL+SHIFT+Key_Z);
 _editMenu->insertSeparator();
 _editMenu->insertItem("Cut", this, SLOT(onCut()), CTRL+Key_X);
 _editMenu->insertItem("Copy", this, SLOT(onCopy()), CTRL+Key_C);
 _editMenu->insertItem("Paste", this, SLOT(onPaste()), CTRL+Key_V);
 
 menuBar()->insertItem( "&Edit", _editMenu);
 
 
 
 _toolMenu = new QPopupMenu(this);
 _toolMenu->setCheckable(true);
 _toolMenu->insertItem("0.05", this, SLOT(onToolStrength(int)), 0, 0);
 _toolMenu->insertItem("0.10", this, SLOT(onToolStrength(int)), 0, 1);
 _toolMenu->insertItem("0.15", this, SLOT(onToolStrength(int)), 0, 2);
 _toolMenu->insertItem("0.20", this, SLOT(onToolStrength(int)), 0, 3);
 _toolMenu->insertItem("0.25", this, SLOT(onToolStrength(int)), 0, 4);
 _toolMenu->insertItem("0.30", this, SLOT(onToolStrength(int)), 0, 5);
 _toolMenu->insertItem("0.35", this, SLOT(onToolStrength(int)), 0, 6);
 _toolMenu->insertItem("0.40", this, SLOT(onToolStrength(int)), 0, 7);
 _toolMenu->insertItem("0.45", this, SLOT(onToolStrength(int)), 0, 8);
 _toolMenu->insertItem("0.50", this, SLOT(onToolStrength(int)), 0, 9);
 _toolMenu->insertItem("0.55", this, SLOT(onToolStrength(int)), 0, 10);
 _toolMenu->insertItem("0.60", this, SLOT(onToolStrength(int)), 0, 11);
 _toolMenu->insertItem("0.65", this, SLOT(onToolStrength(int)), 0, 12);
 _toolMenu->insertItem("0.70", this, SLOT(onToolStrength(int)), 0, 13);
 _toolMenu->insertItem("0.75", this, SLOT(onToolStrength(int)), 0, 14);
 _toolMenu->insertItem("0.80", this, SLOT(onToolStrength(int)), 0, 15);
 _toolMenu->insertItem("0.85", this, SLOT(onToolStrength(int)), 0, 16);
 _toolMenu->insertItem("0.90", this, SLOT(onToolStrength(int)), 0, 17);
 _toolMenu->insertItem("0.95", this, SLOT(onToolStrength(int)), 0, 18);
 _toolMenu->insertItem("1.00", this, SLOT(onToolStrength(int)), 0, 19);
 _toolMenu->setItemChecked(0, true);
 menuBar()->insertItem( "&Tool Strength", _toolMenu);
 

 _toolBar = new QToolBar(this, "File Operations");
 _toolBar->setLabel(tr( "File Operations" ));
 
 _newDocumentButton = new QToolButton(QPixmap((const char **)iconnew), tooltip_newdocument, QString::null, this, SLOT(onDocumentNew()), _toolBar);
 _newDocumentButton->setUsesBigPixmap(true);
 _openFileButton = new QToolButton(QPixmap((const char **)iconopen), tooltip_opendocument, QString::null, this, SLOT(onDocumentOpen()), _toolBar);
 _openFileButton->setUsesBigPixmap(true);
 _saveFileButton = new QToolButton(QPixmap((const char **)iconsave), tooltip_savedocument, QString::null, this, SLOT(onDocumentSave()), _toolBar);
 _saveFileButton->setUsesBigPixmap(true);
 _undoButton = new QToolButton(QPixmap((const char **)iconundo), tooltip_undo, QString::null, this, SLOT(onUndo()), _toolBar);
 _undoButton->setUsesBigPixmap(true);
 _redoButton = new QToolButton(QPixmap((const char **)iconredo), tooltip_redo, QString::null, this, SLOT(onRedo()), _toolBar);
 _redoButton->setUsesBigPixmap(true);
 
  
 _toolBar = new QToolBar(QString("Selection and Zoom"), this);
 _toolBar->setVerticalStretchable(true);
 
 _selectSelectButton = new QToolButton(QPixmap((const char **)iconsel), tooltip_select, QString::null, this, SLOT(onSelect()), _toolBar);
 _selectSelectButton->setUsesBigPixmap(true);
 _selectSelectButton->setDown(true);
 
 _selectZoomButton = new QToolButton(QPixmap((const char **)iconzoom), tooltip_zoom, QString::null, this, SLOT(onZoom()), _toolBar);
 _selectZoomButton->setUsesBigPixmap(true);
 
 
  
  
 _toolBar = new QToolBar(QString("Rigid Transformations"), this);
 _toolBar->setVerticalStretchable(true);
   
 _selectMoveButton = new QToolButton(QPixmap((const char **)iconmove), tooltip_move, QString::null, this, SLOT(onMove()), _toolBar);
 _selectMoveButton->setUsesBigPixmap(true);
  
 _selectRotateButton = new QToolButton(QPixmap((const char **)iconrot), tooltip_rotate, QString::null, this, SLOT(onRotate()), _toolBar);
 _selectRotateButton->setUsesBigPixmap(true);
  
 _selectScaleButton = new QToolButton(QPixmap((const char **)iconscale), tooltip_scale, QString::null, this, SLOT(onScale()), _toolBar);
 _selectScaleButton->setUsesBigPixmap(true);
 
 
 
 _toolBar = new QToolBar(QString("Vertex Transformations"), this);
 _toolBar->setVerticalStretchable(true);
 
 _selectSelectVertexButton = new QToolButton(QPixmap((const char **)iconselvertex), tooltip_selectvertices, QString::null, this, SLOT(onSelectVertex()), _toolBar);
 _selectSelectVertexButton->setUsesBigPixmap(true);
   
 _selectMoveVertexButton = new QToolButton(QPixmap((const char **)iconmovevertex), tooltip_movevertices, QString::null, this, SLOT(onMoveVertex()), _toolBar);
 _selectMoveVertexButton->setUsesBigPixmap(true);
 
 _selectRotateVertexButton = new QToolButton(QPixmap((const char **)iconrotatevertex), tooltip_rotatevertices, QString::null, this, SLOT(onRotateVertex()), _toolBar);
 _selectRotateVertexButton->setUsesBigPixmap(true);

 _selectScaleVertexButton = new QToolButton(QPixmap((const char **)iconscalevertex), tooltip_scalevertices, QString::null, this, SLOT(onScaleVertex()), _toolBar);
 _selectScaleVertexButton->setUsesBigPixmap(true);
 
 _selectMagnetButton = new QToolButton(QPixmap((const char **)iconmagnet), tooltip_magnetictool, QString::null, this, SLOT(onMagnet()), _toolBar);
 _selectMagnetButton->setUsesBigPixmap(true);
 
 
 _selectCreateVertexButton = new QToolButton(QPixmap((const char **)iconaddvertex), tooltip_createvertex, QString::null, this, SLOT(onCreateVertex()), _toolBar);
 _selectCreateVertexButton->setUsesBigPixmap(true);

 _selectCreateTriangleButton = new QToolButton(QPixmap((const char **)iconaddtriangle), tooltip_createtriangle, QString::null, this, SLOT(onCreateTriangle()), _toolBar);
 _selectCreateTriangleButton->setUsesBigPixmap(true);
 
 _selectFlipEdgeButton = new QToolButton(QPixmap((const char **)iconflipedge), tooltip_flipedge, QString::null, this, SLOT(onFlipEdge()), _toolBar);
 _selectFlipEdgeButton->setUsesBigPixmap(true);
 
 _selectSnapToMasterButton = new QToolButton(QPixmap((const char **)iconsnapvertex2master), tooltip_snap2master, QString::null, this, SLOT(onSnapToMaster()), _toolBar);
 _selectSnapToMasterButton->setUsesBigPixmap(true);
 
 
  
 _toolBar = new QToolBar(QString("View Select"), this);
 _toolBar->setVerticalStretchable(true);

 _select3DViewButton = new QToolButton(QPixmap((const char **)icon3d), "3D View", QString::null, this, SLOT(on3DView()), _toolBar);
 _select3DViewButton->setUsesBigPixmap(true);

 _selectXYViewButton = new QToolButton(QPixmap((const char **)iconxy), "XY View", QString::null, this, SLOT(onXYView()), _toolBar);
 _selectXYViewButton->setUsesBigPixmap(true);

 _selectYZViewButton = new QToolButton(QPixmap((const char **)iconyz), "YZ View", QString::null, this, SLOT(onYZView()), _toolBar);
 _selectYZViewButton->setUsesBigPixmap(true);

 _selectZXViewButton = new QToolButton(QPixmap((const char **)iconzx), "ZX View", QString::null, this, SLOT(onZXView()), _toolBar);
 _selectZXViewButton->setUsesBigPixmap(true);

 _selectQuadViewButton = new QToolButton(QPixmap((const char **)iconquad), "Quad View", QString::null, this, SLOT(onQuadView()), _toolBar);
 _selectQuadViewButton->setUsesBigPixmap(true);
 _selectQuadViewButton->setDown(true);

 _selectTextureViewButton = new QToolButton(QPixmap((const char **)icontex), "Texture View", QString::null, this, SLOT(onTextureView()), _toolBar);
 _selectTextureViewButton->setUsesBigPixmap(true);
 
 
 /// The information toolbar..
 _toolBar = new QToolBar(QString("Information"), this, QMainWindow::Bottom);
 _toolBar->setVerticalStretchable(true);
 _toolBar->setHorizontalStretchable(false);
 new QLabel("   Position: ", _toolBar);
 _positionDisplay = new QLabel("0.000000 0.000000 0.000000", _toolBar);
 
 
 // Create the central views, model view, structure view and property view. 
 centralSplitter = new QSplitter(QSplitter::Horizontal, this);
 centralSplitter->setOpaqueResize(true);
 
 setCentralWidget(centralSplitter);

 QSplitter *structureSplitter = new QSplitter(QSplitter::Vertical, centralSplitter);

 structureSplitter->setOpaqueResize(true);

 MStructureView::create(structureSplitter);
 
 structureSplitter->setResizeMode(MStructureView::get(), QSplitter::Stretch);

 centralSplitter->setResizeMode(structureSplitter, QSplitter::Stretch);

 MStructureView::get()->setMinimumSize(200, 480);
 structureSplitter->setResizeMode(MStructureView::get(), QSplitter::Stretch);

 MModelView::create(centralSplitter);
 MModelView::get()->setMinimumSize(640, 480);
 MModelView::get()->setBackgroundColor(QColor(255, 255, 255));
 
 centralSplitter->refresh(); 
 setHistoryButtonState(false, false);
 
 setActiveFunction(Select);
 updateToolbar();
 
 
 _toolStrength = 0.05;
}


MMainWindow *MMainWindow::get(void)
{
 if(_this == NULL) _this = new MMainWindow("moth Version 1 / Release 1");
 return _this;
}


void MMainWindow::setPositionDisplay(QString contents)
{
 _positionDisplay->setText(contents);
}


MMainWindow::~MMainWindow(void)
{

}


void MMainWindow::setHistoryButtonState(bool undoState, bool redoState)
{
 _undoButton->setEnabled(undoState);
 _redoButton->setEnabled(redoState);
} 


void MMainWindow::onProjectNew(void)
{
 QString dirname;
 QString initialDir;
 QDir directory;
 
 initialDir = getenv("HOME");
 if(QMessageBox::warning(this, "Confirm", "Do you really wish to discard the current scene?", QMessageBox::Yes, QMessageBox::No, QMessageBox::NoButton) == QMessageBox::Yes)
 {
  dirname = QFileDialog::getExistingDirectory(initialDir, this, "get existing directory", "Choose a directory", TRUE);
  if(!dirname.isNull() && !dirname.isEmpty())
  {
   // create the project dir.
 	 directory.setPath(dirname);
 	 directory.cd(dirname);
	 directory.mkdir("scenes");
	 directory.mkdir("textures");
	 directory.mkdir("models");
	
	 MDocument::get()->clear();

   _undoButton->setEnabled(false);
   _redoButton->setEnabled(false);

   MStructureView::get()->update();
   MModelView::get()->repaint();
	
	 MPreferences::get()->setProjectLocation(directory.path());
	 MPreferences::get()->writePreferences();
  }
 }
}


void MMainWindow::onDocumentNew(void)
{
 if(QMessageBox::warning(this, "Confirm", "Do you really wish to discard the current scene?", QMessageBox::Yes, QMessageBox::No, QMessageBox::NoButton) == QMessageBox::Yes)
 {
  MDocument::get()->clear();

  _undoButton->setEnabled(false);
  _redoButton->setEnabled(false);

  MStructureView::get()->update();
  MModelView::get()->repaint();
 }
}


void MMainWindow::onDocumentOpen(void)
{
 QString filename;
 QString initialDir;
 
 initialDir = (MPreferences::get()->getProjectLocation()) + "/scenes";
  
 filename = QFileDialog::getOpenFileName(initialDir, QString("*.mscn *.MSCN\n*.*"));
 
 MDocument::get()->commit();
 MDocument::get()->loadScene(filename);
  
 MStructureView::get()->update();
 MModelView::get()->repaint();
}


void MMainWindow::onDocumentSave(void)
{
 QFile QHFile;
 QTextStream os;
 QString warningText;
 QString extension1(".mscn");
 QString extension2(".MSCN");
 QString filename;
 
 if(MPreferences::get()->getDocumentFile() == QString("none"))
 {
  onDocumentSaveAs();
 }
 else
 {
  filename = MPreferences::get()->getDocumentFile();
  if(!(filename.endsWith(extension1) || filename.endsWith(extension2)))
  {
   filename += ".mscn";
  }
 
  QHFile.setName(filename);
  if(QHFile.exists())
  {
   warningText = "The file:\n"; 
   warningText += MPreferences::get()->getDocumentFile();
   warningText += "\nexists!";
   
   if(QMessageBox::warning(this, "File exists", warningText, "Replace", "Cancel", NULL, 0, -1) > 0)
   {
    return;
   }   
  }
  
  if(!QHFile.open(IO_WriteOnly))
  {
   MPreferences::get()->setDocumentFile("none");
   QMessageBox::critical(this, "File IO Error", "The file you wish to save cannot be written.\nYou possibly have no write permissions in the\nselected directory or maybe the disk is full.\nIf you cannot figure out the problem right now,\nplease terminate the session immediately and\ncheck your entire system for malfunctions.", "OK");
  }
  else
  {
   os.setDevice(&QHFile); 
   os << MSceneGraph::get()->toString();
   QHFile.close();
   
   if(QHFile.status() != IO_Ok)
   {
    QMessageBox::critical(this, "File IO Error", "The file you wish to save cannot be written.\nYou possibly have no write permissions in the\nselected directory or maybe the disk is full.\nIf you cannot figure out the problem right now,\nplease terminate the session immediately and\ncheck your entire system for malfunctions.", "OK");
    QHFile.remove();
   }
  }
 }
}


void MMainWindow::onDocumentSaveAs(void)
{
 QString filename;
 QString initialDir;
 QString fileFilter("*.mscn *.MSCN");
 
 initialDir = (MPreferences::get()->getProjectLocation()) + "/scenes";
 
 filename = QFileDialog::getSaveFileName(initialDir, fileFilter);
 
 if(!filename.isNull() && !filename.isEmpty())
 {
  MPreferences::get()->setDocumentFile(filename);
  onDocumentSave();
 }
}


void MMainWindow::onPreferences(void)
{
 MPreferencesDialog prefs;

 if(prefs.exec())
 {
  MMaterialManager::get()->setProjectDirectory(MPreferences::get()->getProjectLocation());
 }
}



void MMainWindow::onQuit(void)
{
 if(QMessageBox::warning(this, "Confirm", "Do you really wish to quit?", QMessageBox::Yes, QMessageBox::No, QMessageBox::NoButton) == QMessageBox::Yes)
 {
  qApp->quit();
 }
}


void MMainWindow::onUndo(void)
{
 MItemManipulator::get()->clearVertexStack();

 MDocument::get()->undo();
 
 MStructureView::get()->update();
 MModelView::get()->repaint();
}


void MMainWindow::onRedo(void)
{
 MItemManipulator::get()->clearVertexStack();
 
 MDocument::get()->redo();
 
 MStructureView::get()->update();
 MModelView::get()->repaint();
}



void MMainWindow::onCut(void)
{
 MItemManipulator::get()->clearVertexStack();

 MStructureView::get()->onCut();
 MModelView::get()->repaint();
}


void MMainWindow::onCopy(void)
{
 MItemManipulator::get()->clearVertexStack();

 MStructureView::get()->onCopy();
 MModelView::get()->repaint();
}


void MMainWindow::onPaste(void)
{
 MItemManipulator::get()->clearVertexStack();

 MStructureView::get()->onPaste();
 MModelView::get()->repaint();
}


void MMainWindow::onToolStrength(int index)
{
 int i;

 for(i = 0; i < 20; i++)
 {
  _toolMenu->setItemChecked(i, false);  
 }
 _toolMenu->setItemChecked(index, true);

 _toolStrength = 0.05 + ((float)index)/20.0;
}


void MMainWindow::onSelect(void)
{
 MItemManipulator::get()->clearVertexStack();

 setActiveFunction(Select);
 MModelView::get()->repaint();
}


void MMainWindow::onZoom(void)
{
 MItemManipulator::get()->clearVertexStack();

 setActiveFunction(Zoom);
 MModelView::get()->repaint();
}



void MMainWindow::onMove(void)
{
 MItemManipulator::get()->clearVertexStack();

 setActiveFunction(Move);
 MModelView::get()->repaint();
}


void MMainWindow::onRotate(void)
{
 MItemManipulator::get()->clearVertexStack();
 
 setActiveFunction(Rotate);
 MModelView::get()->repaint();
}


void MMainWindow::onScale(void)
{
 MItemManipulator::get()->clearVertexStack();
 
 setActiveFunction(Scale);
 MModelView::get()->repaint();
}

void MMainWindow::onSelectVertex(void)
{
 MItemManipulator::get()->clearVertexStack();
 
 setActiveFunction(SelectVertex);
 MModelView::get()->repaint();
}


void MMainWindow::onCreateVertex(void)
{
 MItemManipulator::get()->clearVertexStack();
 
 setActiveFunction(CreateVertex);
 MModelView::get()->repaint();
}


void MMainWindow::onCreateTriangle(void)
{
 MItemManipulator::get()->clearVertexStack();
 
 setActiveFunction(CreateTriangle);
 MModelView::get()->repaint();
}


void MMainWindow::onMoveVertex(void)
{
 setActiveFunction(MoveVertex);
 MModelView::get()->repaint();
}


void MMainWindow::onRotateVertex(void)
{
 setActiveFunction(RotateVertex);
 MModelView::get()->repaint();
}


void MMainWindow::onScaleVertex(void)
{
 setActiveFunction(ScaleVertex);
 MModelView::get()->repaint();
}


void MMainWindow::onMagnet(void)
{
 setActiveFunction(Magnet);
 MModelView::get()->repaint();
}


void MMainWindow::onFlipEdge(void)
{
 MItemManipulator::get()->clearVertexStack();

 setActiveFunction(FlipEdge);
 MModelView::get()->repaint();
}


void MMainWindow::onSnapToMaster(void)
{
 MItemManipulator::get()->clearVertexStack();
 
 setActiveFunction(SnapToMaster);
 MModelView::get()->repaint();
}


void MMainWindow::on3DView(void)
{
 _select3DViewButton->setDown(true);
 _selectXYViewButton->setDown(false);
 _selectYZViewButton->setDown(false);
 _selectZXViewButton->setDown(false);
 _selectQuadViewButton->setDown(false);
 _selectTextureViewButton->setDown(false);
 
 MModelView::get()->select3DView();
}


void MMainWindow::onXYView(void)
{
 _select3DViewButton->setDown(false);
 _selectXYViewButton->setDown(true);
 _selectYZViewButton->setDown(false);
 _selectZXViewButton->setDown(false);
 _selectQuadViewButton->setDown(false);
 _selectTextureViewButton->setDown(false);
 
 MModelView::get()->selectXYView();
}


void MMainWindow::onYZView(void)
{
 _select3DViewButton->setDown(false);
 _selectXYViewButton->setDown(false);
 _selectYZViewButton->setDown(true);
 _selectZXViewButton->setDown(false);
 _selectQuadViewButton->setDown(false);
 _selectTextureViewButton->setDown(false);
  
 MModelView::get()->selectYZView();
}


void MMainWindow::onZXView(void)
{
 _select3DViewButton->setDown(false);
 _selectXYViewButton->setDown(false);
 _selectYZViewButton->setDown(false);
 _selectZXViewButton->setDown(true);
 _selectQuadViewButton->setDown(false);
 _selectTextureViewButton->setDown(false);
  
 MModelView::get()->selectZXView();
}


void MMainWindow::onQuadView(void)
{
 _select3DViewButton->setDown(false);
 _selectXYViewButton->setDown(false);
 _selectYZViewButton->setDown(false);
 _selectZXViewButton->setDown(false);
 _selectQuadViewButton->setDown(true);
 _selectTextureViewButton->setDown(false);
  
 MModelView::get()->selectQuadView();
}


void MMainWindow::onTextureView(void)
{
 _select3DViewButton->setDown(false);
 _selectXYViewButton->setDown(false);
 _selectYZViewButton->setDown(false);
 _selectZXViewButton->setDown(false);
 _selectQuadViewButton->setDown(false);
 _selectTextureViewButton->setDown(true);
 
 MModelView::get()->selectTextureView();
}



void MMainWindow::onExport(int index)
{
 
}


void MMainWindow::onImport(int index)
{
 
}


void MMainWindow::onAbout(void)
{
 QString abouttext = 
 "<b>moth Version 1 / Release 1</b>"
 "<blockquote>"
 "(C) 2002 - 2005 by Peter Uray"
 "</blockquote>"
 "<blockquote>"
 "For detailed legal information please see the manual "
 "or read the README file distributed with this program. "
 "</blockquote>";

  QMessageBox _aboutDlg("About moth", abouttext, QMessageBox::Information, QMessageBox::Ok, 0, 0, 0, 0, TRUE);
  
 _aboutDlg.exec();
}


void MMainWindow::setActiveFunction(MFunction function)
{
 _activeFunction = function;
 
 
 _selectSelectButton->setDown(false);
 _selectZoomButton->setDown(false);
 
 _selectMoveButton->setDown(false);
 _selectRotateButton->setDown(false);
 _selectScaleButton->setDown(false);
 
 _selectSelectVertexButton->setDown(false);
 _selectCreateVertexButton->setDown(false);
 _selectCreateTriangleButton->setDown(false);
 _selectMoveVertexButton->setDown(false);
 _selectRotateVertexButton->setDown(false);
 _selectScaleVertexButton->setDown(false);
 _selectMagnetButton->setDown(false);
 _selectFlipEdgeButton->setDown(false);
 _selectSnapToMasterButton->setDown(false);
 
 switch(function)
 {
  case Select: _selectSelectButton->setDown(true); break;
  case Zoom: _selectZoomButton->setDown(true); break;
  case Move: _selectMoveButton->setDown(true); break;
  case Rotate: _selectRotateButton->setDown(true); break;
  case Scale: _selectScaleButton->setDown(true); break;
  case SelectVertex: _selectSelectVertexButton->setDown(true); break;
  case CreateVertex: _selectCreateVertexButton->setDown(true); break;
  case CreateTriangle: _selectCreateTriangleButton->setDown(true); break;
  case MoveVertex: _selectMoveVertexButton->setDown(true); break;
	case Magnet: _selectMagnetButton->setDown(true); break;

	case RotateVertex: _selectRotateVertexButton->setDown(true); break;
	case ScaleVertex: _selectScaleVertexButton->setDown(true); break;

  case FlipEdge: _selectFlipEdgeButton->setDown(true); break;
  case SnapToMaster: _selectSnapToMasterButton->setDown(true); break;
  default: break;
 }
}


MMainWindow::MFunction MMainWindow::getActiveFunction(void)
{
 return _activeFunction;
}


float MMainWindow::getToolStrength(void)
{
 return _toolStrength;
}


void MMainWindow::updateToolbar(void)
{
 MSceneGraphItem *item;
  
 item = MDocument::get()->getSelectedItem();
 
 if(!item)
 {
  _selectSelectButton->setEnabled(true);
  _selectZoomButton->setEnabled(true);
 
  _selectMoveButton->setEnabled(false);
  _selectRotateButton->setEnabled(false);
  _selectScaleButton->setEnabled(false);

  _selectSelectVertexButton->setEnabled(false);
  _selectCreateVertexButton->setEnabled(false);
  _selectCreateTriangleButton->setEnabled(false);
  
	_selectMoveVertexButton->setEnabled(false);
	_selectRotateVertexButton->setEnabled(false);
	_selectScaleVertexButton->setEnabled(false);
	_selectMagnetButton->setEnabled(false);
	
  _selectFlipEdgeButton->setEnabled(false);
  _selectSnapToMasterButton->setEnabled(false);
  
  _selectTextureViewButton->setEnabled(false);
  
  return;
 }
 
 switch(item->getType())
 {
  case MEnum::Model:
  case MEnum::LightSource:
  
    _selectSelectButton->setEnabled(true);
    _selectZoomButton->setEnabled(true);
 
    _selectMoveButton->setEnabled(true);
    _selectRotateButton->setEnabled(true);
    _selectScaleButton->setEnabled(true);

    _selectSelectVertexButton->setEnabled(false);
    _selectCreateVertexButton->setEnabled(false);
    _selectCreateTriangleButton->setEnabled(false);
    _selectMoveVertexButton->setEnabled(false);
		
		_selectRotateVertexButton->setEnabled(false);
		_selectScaleVertexButton->setEnabled(false);
		_selectMagnetButton->setEnabled(false);
		
    _selectFlipEdgeButton->setEnabled(false);
    _selectSnapToMasterButton->setEnabled(false);
    
    _selectTextureViewButton->setEnabled(false);
  
  break;
  
  case MEnum::Shape:
  
    _selectSelectButton->setEnabled(true);
    _selectZoomButton->setEnabled(true);
 
    _selectMoveButton->setEnabled(true);
    _selectRotateButton->setEnabled(true);
    _selectScaleButton->setEnabled(true);

    _selectSelectVertexButton->setEnabled(true);
    _selectCreateVertexButton->setEnabled(true);
    _selectCreateTriangleButton->setEnabled(true);
    _selectMoveVertexButton->setEnabled(true);
		
		_selectRotateVertexButton->setEnabled(true);
		_selectScaleVertexButton->setEnabled(true);
		_selectMagnetButton->setEnabled(true);
		
    _selectFlipEdgeButton->setEnabled(true);
    _selectSnapToMasterButton->setEnabled(true);
    
    _selectTextureViewButton->setEnabled(true);
  
  break;
  
 
  default:
  
  break;
 }
}
