
/*
 This file is part of moth. moth is a program for creating
 and editing textured 3D models.
 
 Copyright (C) 2004 Peter Uray.

 moth is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 moth is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with moth; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <iostream>

#include <qvariant.h>
#include <qhbox.h>
#include <qlayout.h>
#include <qlabel.h>

#include <moth/MMaterialManager.h>

#include <MDocument.h>
#include <MModelView.h>
#include <MMaterialDialog.h>



MMaterialDialog::MMaterialDialog(void)
{
 QGridLayout* gl;
 QVariant value;
 float *params;
 int r, g, b, a;
 float shininess;
 QStringList textureList;
 
 setCaption("Material Parameters");
 
 resize(800, 240);

 gl = new QGridLayout(this, 9, 30, 3);
 
 /// color
 gl->addMultiCellWidget(new QLabel("Color", this), 0, 0, 0, 3);
 
 _colorDisplay = new QButton(this);
 
 MMaterial &material = MDocument::get()->getSelectedShape()->getMaterial();
 
 params = material.getColor();
 r = (int)(255.0*params[0]);
 g = (int)(255.0*params[1]);
 b = (int)(255.0*params[2]);
 a = (int)(255.0*params[3]);
 _colorDisplay->setPaletteBackgroundColor(QColor(r, g, b));
 gl->addMultiCellWidget(_colorDisplay, 1, 1, 0, 3);
 
 _colorRSlider = new QSlider(Qt::Vertical, this);
 _colorRSlider->setMinValue(0);
 _colorRSlider->setMaxValue(255);
 _colorRSlider->setValue(255 - r);
 connect(_colorRSlider, SIGNAL(valueChanged(int)), this, SLOT(onSliderValueChanged(int)));
 gl->addMultiCellWidget(_colorRSlider, 2, 6, 0, 0);
 gl->addMultiCellWidget(new QLabel("R", this), 7, 7, 0, 0);
 
 _colorGSlider = new QSlider(Qt::Vertical, this);
 _colorGSlider->setMinValue(0);
 _colorGSlider->setMaxValue(255);
 _colorGSlider->setValue(255 - g);
 connect(_colorGSlider, SIGNAL(valueChanged(int)), this, SLOT(onSliderValueChanged(int)));
 gl->addMultiCellWidget(_colorGSlider, 2, 6, 1, 1);
 gl->addMultiCellWidget(new QLabel("G", this), 7, 7, 1, 1);
 
 _colorBSlider = new QSlider(Qt::Vertical, this);
 _colorBSlider->setMinValue(0);
 _colorBSlider->setMaxValue(255);
 _colorBSlider->setValue(255 - b);
 connect(_colorBSlider, SIGNAL(valueChanged(int)), this, SLOT(onSliderValueChanged(int)));
 gl->addMultiCellWidget(_colorBSlider, 2, 6, 2, 2);
 gl->addMultiCellWidget(new QLabel("B", this), 7, 7, 2, 2);
 
 _colorASlider = new QSlider(Qt::Vertical, this);
 _colorASlider->setMinValue(0);
 _colorASlider->setMaxValue(255);
 _colorASlider->setValue(255 - a);
 connect(_colorASlider, SIGNAL(valueChanged(int)), this, SLOT(onSliderValueChanged(int)));
 gl->addMultiCellWidget(_colorASlider, 2, 6, 3, 3);
 gl->addMultiCellWidget(new QLabel("A", this), 7, 7, 3, 3);


 gl->setColStretch(4, 1); 




 /// ambient color
 gl->addMultiCellWidget(new QLabel("Ambient", this), 0, 0, 5, 8);
 
 _ambientDisplay = new QButton(this);
 
 params = material.getAmbient();
 r = (int)(255.0*params[0]);
 g = (int)(255.0*params[1]);
 b = (int)(255.0*params[2]);
 a = (int)(255.0*params[3]);
 _ambientDisplay->setPaletteBackgroundColor(QColor(r, g, b));
 gl->addMultiCellWidget(_ambientDisplay, 1, 1, 5, 8);
 
 _ambientRSlider = new QSlider(Qt::Vertical, this);
 _ambientRSlider->setMinValue(0);
 _ambientRSlider->setMaxValue(255);
 _ambientRSlider->setValue(255 - r);
 connect(_ambientRSlider, SIGNAL(valueChanged(int)), this, SLOT(onSliderValueChanged(int)));
 gl->addMultiCellWidget(_ambientRSlider, 2, 6, 5, 5);
 gl->addMultiCellWidget(new QLabel("R", this), 7, 7, 5, 5);
 
 _ambientGSlider = new QSlider(Qt::Vertical, this);
 _ambientGSlider->setMinValue(0);
 _ambientGSlider->setMaxValue(255);
 _ambientGSlider->setValue(255 - g);
 connect(_ambientGSlider, SIGNAL(valueChanged(int)), this, SLOT(onSliderValueChanged(int)));
 gl->addMultiCellWidget(_ambientGSlider, 2, 6, 6, 6);
 gl->addMultiCellWidget(new QLabel("G", this), 7, 7, 6, 6);
 
 _ambientBSlider = new QSlider(Qt::Vertical, this);
 _ambientBSlider->setMinValue(0);
 _ambientBSlider->setMaxValue(255);
 _ambientBSlider->setValue(255 - b);
 connect(_ambientBSlider, SIGNAL(valueChanged(int)), this, SLOT(onSliderValueChanged(int)));
 gl->addMultiCellWidget(_ambientBSlider, 2, 6, 7, 7);
 gl->addMultiCellWidget(new QLabel("B", this), 7, 7, 7, 7);
 
 _ambientASlider = new QSlider(Qt::Vertical, this);
 _ambientASlider->setMinValue(0);
 _ambientASlider->setMaxValue(255);
 _ambientASlider->setValue(255 - a);
 connect(_ambientASlider, SIGNAL(valueChanged(int)), this, SLOT(onSliderValueChanged(int)));
 gl->addMultiCellWidget(_ambientASlider, 2, 6, 8, 8);
 gl->addMultiCellWidget(new QLabel("A", this), 7, 7, 8, 8);
 


 gl->setColStretch(9, 1); 


 
 /// diffuse color
 gl->addMultiCellWidget(new QLabel("Diffuse", this), 0, 0, 10, 13);
 
 _diffuseDisplay = new QButton(this);
 
 params = material.getDiffuse();
 r = (int)(255.0*params[0]);
 g = (int)(255.0*params[1]);
 b = (int)(255.0*params[2]);
 a = (int)(255.0*params[3]);
 
 _diffuseDisplay->setPaletteBackgroundColor(QColor(r, g, b));
 gl->addMultiCellWidget(_diffuseDisplay, 1, 1, 10, 13);
 
 _diffuseRSlider = new QSlider(Qt::Vertical, this);
 _diffuseRSlider->setMinValue(0);
 _diffuseRSlider->setMaxValue(255);
 _diffuseRSlider->setValue(255 - r);
 connect(_diffuseRSlider, SIGNAL(valueChanged(int)), this, SLOT(onSliderValueChanged(int)));
 gl->addMultiCellWidget(_diffuseRSlider, 2, 6, 10, 10);
 gl->addMultiCellWidget(new QLabel("R", this), 7, 7, 10, 10);
 
 _diffuseGSlider = new QSlider(Qt::Vertical, this);
 _diffuseGSlider->setMinValue(0);
 _diffuseGSlider->setMaxValue(255);
 _diffuseGSlider->setValue(255 - g);
 connect(_diffuseGSlider, SIGNAL(valueChanged(int)), this, SLOT(onSliderValueChanged(int)));
 gl->addMultiCellWidget(_diffuseGSlider, 2, 6, 11, 11);
 gl->addMultiCellWidget(new QLabel("G", this), 7, 7, 11, 11);
 
 _diffuseBSlider = new QSlider(Qt::Vertical, this);
 _diffuseBSlider->setMinValue(0);
 _diffuseBSlider->setMaxValue(255);
 _diffuseBSlider->setValue(255 - b);
 connect(_diffuseBSlider, SIGNAL(valueChanged(int)), this, SLOT(onSliderValueChanged(int)));
 gl->addMultiCellWidget(_diffuseBSlider, 2, 6, 12, 12);
 gl->addMultiCellWidget(new QLabel("B", this), 7, 7, 12, 12);

 _diffuseASlider = new QSlider(Qt::Vertical, this);
 _diffuseASlider->setMinValue(0);
 _diffuseASlider->setMaxValue(255);
 _diffuseASlider->setValue(255 - a);
 connect(_diffuseASlider, SIGNAL(valueChanged(int)), this, SLOT(onSliderValueChanged(int)));
 gl->addMultiCellWidget(_diffuseASlider, 2, 6, 13, 13);
 gl->addMultiCellWidget(new QLabel("A", this), 7, 7, 13, 13);
 
 
 
 gl->setColStretch(14, 1); 
 
 
 
 
 /// specular color
 gl->addMultiCellWidget(new QLabel("Specular", this), 0, 0, 15, 18);
 
 _specularDisplay = new QButton(this);
 
 params = material.getSpecular();
 r = (int)(255.0*params[0]);
 g = (int)(255.0*params[1]);
 b = (int)(255.0*params[2]);
 a = (int)(255.0*params[3]);
 
 _specularDisplay->setPaletteBackgroundColor(QColor(r, g, b));
 gl->addMultiCellWidget(_specularDisplay, 1, 1, 15, 18);
  
 _specularRSlider = new QSlider(Qt::Vertical, this);
 _specularRSlider->setMinValue(0);
 _specularRSlider->setMaxValue(255);
 _specularRSlider->setValue(255 - r);
 connect(_specularRSlider, SIGNAL(valueChanged(int)), this, SLOT(onSliderValueChanged(int)));
 gl->addMultiCellWidget(_specularRSlider, 2, 6, 15, 15);
 gl->addMultiCellWidget(new QLabel("R", this), 7, 7, 15, 15);
 
 _specularGSlider = new QSlider(Qt::Vertical, this);
 _specularGSlider->setMinValue(0);
 _specularGSlider->setMaxValue(255);
 _specularGSlider->setValue(255 - g);
 connect(_specularGSlider, SIGNAL(valueChanged(int)), this, SLOT(onSliderValueChanged(int)));
 gl->addMultiCellWidget(_specularGSlider, 2, 6, 16, 16);
 gl->addMultiCellWidget(new QLabel("G", this), 7, 7, 16, 16);
 
 _specularBSlider = new QSlider(Qt::Vertical, this);
 _specularBSlider->setMinValue(0);
 _specularBSlider->setMaxValue(255);
 _specularBSlider->setValue(255 - b);
 connect(_specularBSlider, SIGNAL(valueChanged(int)), this, SLOT(onSliderValueChanged(int)));
 gl->addMultiCellWidget(_specularBSlider, 2, 6, 17, 17);
 gl->addMultiCellWidget(new QLabel("B", this), 7, 7, 17, 17);
 
 _specularASlider = new QSlider(Qt::Vertical, this);
 _specularASlider->setMinValue(0);
 _specularASlider->setMaxValue(255);
 _specularASlider->setValue(255 - a);
 connect(_specularASlider, SIGNAL(valueChanged(int)), this, SLOT(onSliderValueChanged(int)));
 gl->addMultiCellWidget(_specularASlider, 2, 6, 18, 18);
 gl->addMultiCellWidget(new QLabel("A", this), 7, 7, 18, 18);


 gl->setColStretch(19, 1); 
 
 
 
 /// emission color
 gl->addMultiCellWidget(new QLabel("Emission", this), 0, 0, 20, 23);
 
 _emissionDisplay = new QButton(this);
 
 params = material.getEmission();
 r = (int)(255.0*params[0]);
 g = (int)(255.0*params[1]);
 b = (int)(255.0*params[2]);
 a = (int)(255.0*params[3]);
 
 _emissionDisplay->setPaletteBackgroundColor(QColor(r, g, b));
 gl->addMultiCellWidget(_emissionDisplay, 1, 1, 20, 23);
  
 _emissionRSlider = new QSlider(Qt::Vertical, this);
 _emissionRSlider->setMinValue(0);
 _emissionRSlider->setMaxValue(255);
 _emissionRSlider->setValue(255 - r);
 connect(_emissionRSlider, SIGNAL(valueChanged(int)), this, SLOT(onSliderValueChanged(int)));
 gl->addMultiCellWidget(_emissionRSlider, 2, 6, 20, 20);
 gl->addMultiCellWidget(new QLabel("R", this), 7, 7, 20, 20);
 
 _emissionGSlider = new QSlider(Qt::Vertical, this);
 _emissionGSlider->setMinValue(0);
 _emissionGSlider->setMaxValue(255);
 _emissionGSlider->setValue(255 - g);
 connect(_emissionGSlider, SIGNAL(valueChanged(int)), this, SLOT(onSliderValueChanged(int)));
 gl->addMultiCellWidget(_emissionGSlider, 2, 6, 21, 21);
 gl->addMultiCellWidget(new QLabel("G", this), 7, 7, 21, 21);
 
 _emissionBSlider = new QSlider(Qt::Vertical, this);
 _emissionBSlider->setMinValue(0);
 _emissionBSlider->setMaxValue(255);
 _emissionBSlider->setValue(255 - b);
 connect(_emissionBSlider, SIGNAL(valueChanged(int)), this, SLOT(onSliderValueChanged(int)));
 gl->addMultiCellWidget(_emissionBSlider, 2, 6, 22, 22);
 gl->addMultiCellWidget(new QLabel("B", this), 7, 7, 22, 22);
 
 _emissionASlider = new QSlider(Qt::Vertical, this);
 _emissionASlider->setMinValue(0);
 _emissionASlider->setMaxValue(255);
 _emissionASlider->setValue(255 - a);
 connect(_emissionASlider, SIGNAL(valueChanged(int)), this, SLOT(onSliderValueChanged(int)));
 gl->addMultiCellWidget(_emissionASlider, 2, 6, 23, 23);
 gl->addMultiCellWidget(new QLabel("A", this), 7, 7, 23, 23);
 
 gl->setColStretch(24, 1);



 // shininess
 shininess = material.getShininess();
 gl->addMultiCellWidget(new QLabel("S", this), 0, 0, 25, 25);
 _shininessSlider = new QSlider(Qt::Vertical, this);
 _shininessSlider->setMinValue(0);
 _shininessSlider->setMaxValue(100);
 _shininessSlider->setValue(100 - (int)shininess);
 connect(_shininessSlider, SIGNAL(valueChanged(int)), this, SLOT(onSliderValueChanged(int)));
 gl->addMultiCellWidget(_shininessSlider, 2, 6, 25, 25);


 gl->setColStretch(26, 1);


 // shading model.
 gl->addMultiCellWidget(new QLabel("Shading Model", this), 0, 0, 27, 27);
 _renderModeBox = new QComboBox(this);
 _renderModeBox->insertItem("Flat");
 _renderModeBox->insertItem("Smooth");
 _renderModeBox->insertItem("Metallic");
 gl->addMultiCellWidget(_renderModeBox, 1, 1, 27, 27);
 connect(_renderModeBox, SIGNAL(activated(int)), this, SLOT(onShadingModelChanged(int)));
 switch(material.getShadingModel())
 {
  case MEnum::Flat: _renderModeBox->setCurrentItem(0); break;
  case MEnum::Smooth: _renderModeBox->setCurrentItem(1); break;
  case MEnum::Metallic: _renderModeBox->setCurrentItem(2); break;
 }
 
 _isTransparentBox = new QCheckBox("Transparent", this);
 connect(_isTransparentBox, SIGNAL(toggled(bool)), this, SLOT(onTransparentBoxToggled(bool)));
 gl->addMultiCellWidget(_isTransparentBox, 2, 2, 27, 27);
 _isTransparentBox->setChecked(false);
 if(material.isTransparent()) _isTransparentBox->setChecked(true);
 
 gl->setColStretch(28, 1);

 
 gl->addMultiCellWidget(new QLabel("Texture", this), 0, 0, 29, 29);
 _textureListView = new QListView(this);
 _textureListView->addColumn("Available Textures");
 textureList = MMaterialManager::get()->getTextureList();
 for(QStringList::Iterator it = textureList.begin(); it != textureList.end(); ++it )
 {
  _textureListView->insertItem(new QListViewItem(_textureListView, *it));
 }
 connect(_textureListView, SIGNAL(selectionChanged()), this, SLOT(onTextureSelect()));
 gl->addMultiCellWidget(_textureListView, 1, 6, 29, 29);
 
 gl->setColStretch(29, 10);
 
  
 QHBox *hbox = new QHBox(this);
 _okButton = new QPushButton("Dismiss", hbox);
 connect(_okButton, SIGNAL(released()), this, SLOT(onOKButtonPressed()));
 
 gl->addMultiCellWidget(hbox, 8, 8, 0, 29);
}


MMaterialDialog::~MMaterialDialog(void)
{

}


void MMaterialDialog::onOKButtonPressed(void)
{
 accept();
}


void MMaterialDialog::show(void)
{
 QDialog::show();
}


void MMaterialDialog::onSliderValueChanged(int value)
{
 int r, g, b, a;

 MMaterial &material = MDocument::get()->getSelectedShape()->getMaterial();

 r = 255 - _colorRSlider->value();
 g = 255 - _colorGSlider->value();
 b = 255 - _colorBSlider->value();
 a = 255 - _colorASlider->value();
 _colorDisplay->setPaletteBackgroundColor(QColor(r, g, b));
 material.setAmbient(r/255.0, g/255.0, b/255.0, a/255.0);


 r = 255 - _ambientRSlider->value();
 g = 255 - _ambientGSlider->value();
 b = 255 - _ambientBSlider->value();
 a = 255 - _ambientASlider->value();
 _ambientDisplay->setPaletteBackgroundColor(QColor(r, g, b));
 material.setAmbient(r/255.0, g/255.0, b/255.0, a/255.0);
 
 r = 255 - _diffuseRSlider->value();
 g = 255 - _diffuseGSlider->value();
 b = 255 - _diffuseBSlider->value();
 a = 255 - _diffuseASlider->value();
 _diffuseDisplay->setPaletteBackgroundColor(QColor(r, g, b));
 material.setDiffuse(r/255.0, g/255.0, b/255.0, a/255.0);
 
 r = 255 - _specularRSlider->value();
 g = 255 - _specularGSlider->value();
 b = 255 - _specularBSlider->value();
 a = 255 - _specularASlider->value();
 _specularDisplay->setPaletteBackgroundColor(QColor(r, g, b));
 material.setSpecular(r/255.0, g/255.0, b/255.0, a/255.0);


 r = 255 - _emissionRSlider->value();
 g = 255 - _emissionGSlider->value();
 b = 255 - _emissionBSlider->value();
 a = 255 - _emissionASlider->value();
 _emissionDisplay->setPaletteBackgroundColor(QColor(r, g, b));
 material.setEmission(r/255.0, g/255.0, b/255.0, a/255.0);


 r = 100 - _shininessSlider->value();
 material.setShininess((float)r);

 MModelView::get()->repaint();
}



void MMaterialDialog::onTransparentBoxToggled(bool state)
{
 MMaterial &material = MDocument::get()->getSelectedShape()->getMaterial();
 material.setTransparent(state);
 
 MModelView::get()->repaint();
}


void MMaterialDialog::onShadingModelChanged(int index)
{
 MMaterial &material = MDocument::get()->getSelectedShape()->getMaterial();
 
 switch(index)
 {
  case 0:
    material.setShadingModel(MEnum::Flat);
  break;
  
  case 1:
    material.setShadingModel(MEnum::Smooth);
  break;
  
  case 2:
    material.setShadingModel(MEnum::Metallic);
  break;
  
  default:
    material.setShadingModel(MEnum::Smooth);
  break;
 }
 
 MModelView::get()->repaint();
}


void MMaterialDialog::onTextureSelect(void)
{
 MMaterial &material = MDocument::get()->getSelectedShape()->getMaterial();
 material.setTexture(_textureListView->selectedItem()->text(0));
 
 MModelView::get()->repaint();
}
