
/*
 This file is part of moth. moth is a program for creating
 and editing textured 3D models.
 
 Copyright (C) 2004 Peter Uray.

 moth is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 moth is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with moth; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <qlabel.h>
#include <qlayout.h>
#include <qhbox.h>
#include <qvalidator.h>

#include <MPositionDialog.h>


MPositionDialog::MPositionDialog(QString label) : QDialog(0, "positionDialog", true, Qt::WStyle_DialogBorder | Qt::WStyle_Dialog | Qt::WStyle_StaysOnTop)
{
 QGridLayout* gl;
 
 resize(300, 70);
 setCaption(QString("Enter ") + label);
 
 gl = new QGridLayout(this, 3, 4, 3); 
 
 gl->addWidget(new QLabel(label, this), 0, 0);
 _positionXEdit = new QLineEdit(this);
 _positionXEdit->setAlignment(AlignRight);
 _positionXEdit->setValidator(new QDoubleValidator(-9999999.0, 9999999.0, 2, _positionXEdit));
 gl->addWidget(_positionXEdit, 0, 1);
  
 _positionYEdit = new QLineEdit(this);
 _positionYEdit->setAlignment(AlignRight);
 _positionYEdit->setValidator(new QDoubleValidator(-9999999.0, 9999999.0, 2, _positionYEdit));
 gl->addWidget(_positionYEdit, 0, 2);
  
 _positionZEdit = new QLineEdit(this);
 _positionZEdit->setAlignment(AlignRight);
 _positionZEdit->setValidator(new QDoubleValidator(-9999999.0, 9999999.0, 2, _positionZEdit));
 gl->addWidget(_positionZEdit, 0, 3);

 gl->setRowStretch(1, 10);
  
 QHBox *hbox = new QHBox(this);
 _okButton = new QPushButton("OK", hbox);
 connect(_okButton, SIGNAL(released()), this, SLOT(onOKButtonPressed()));
 
 _cancelButton = new QPushButton("Cancel", hbox);
 connect(_cancelButton, SIGNAL(released()), this, SLOT(reject()));
 gl->addMultiCellWidget(hbox, 2, 2, 0, 3);
}


MPositionDialog::~MPositionDialog(void)
{

}


void MPositionDialog::setPosition(MVector position)
{
 _position = position;
}


MVector MPositionDialog::getPosition(void)
{
 return _position;
}


void MPositionDialog::show(void)
{
 char buffer[80];
 
 sprintf(buffer, "%8.3f", _position[0]);
 _positionXEdit->setText(buffer);
 sprintf(buffer, "%8.3f", _position[1]);
 _positionYEdit->setText(buffer);
 sprintf(buffer, "%8.3f", _position[2]);
 _positionZEdit->setText(buffer);

 QDialog::show();
}


void MPositionDialog::onOKButtonPressed(void)
{
 _position[0] = _positionXEdit->text().toFloat();
 _position[1] = _positionYEdit->text().toFloat();
 _position[2] = _positionZEdit->text().toFloat();

 accept();
}
