
/*
 This file is part of moth. moth is a program for creating
 and editing textured 3D models.
 
 Copyright (C) 2004 Peter Uray.

 moth is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 moth is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with moth; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <iostream>
#include <unistd.h>

#include <qpainter.h>
#include <qpixmap.h>
#include <qapplication.h>

#include <icons/splashscreen.xpm>

#include <MPreferences.h>
#include <MSplashScreen.h>

#define SCREEN_WIDTH	340
#define SCREEN_HEIGHT	340
#define SCREEN_DATA	splashscreen_xpm

MSplashScreen *MSplashScreen::_this = NULL;

MSplashScreen::MSplashScreen(void) : QWidget(NULL, "splash screen", WType_Popup | WStyle_NoBorder)
{
 QPalette palette(black, black);
 
 setGeometry(QApplication::desktop()->width()/2 - SCREEN_WIDTH/2, QApplication::desktop()->height()/2 - SCREEN_HEIGHT/2, SCREEN_WIDTH, SCREEN_HEIGHT);
 
 
 _screen = new QPixmap((const char **)SCREEN_DATA);
 
 _message = "Welcome to Peters Modeller";
 
 _messageEdit = new QLineEdit(this, _message);
 _messageEdit->setGeometry(0, SCREEN_HEIGHT - 20, SCREEN_WIDTH, 20);
 
 setPalette(palette);
}


MSplashScreen::~MSplashScreen(void)
{
 delete _screen;
}



void MSplashScreen::display(const char *message)
{
 if(_this == NULL) return;

 _this->_messageEdit->setText(message);
 _this->_messageEdit->repaint();
 
 QApplication::flush();
}


void MSplashScreen::expose(void)
{
 if(_this == NULL)
 {
  _this = new MSplashScreen();
 }
 _this->show();
 _this->repaint(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT);
}


void MSplashScreen::conceal(void)
{
 if(_this == NULL) return;
 _this->hide();
}


void MSplashScreen::paintEvent(QPaintEvent *)
{
 QPainter p;
 
 p.begin(this);
 p.drawPixmap(0, 0, *_screen);
 p.flush();
 p.end();
 
 _messageEdit->repaint();
}
