
/*
 This file is part of moth. moth is a program for creating
 and editing textured 3D models.
 
 Copyright (C) 2004 Peter Uray.

 moth is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 moth is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with moth; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <qvaluevector.h> 

#include <moth/MShape.h>

#include <MDocument.h>
#include <MSweepRotateCreator.h>


int MSweepRotateCreator::vertexAt(int phi, int theta, int sectors, int rings)
{
 if(theta < 0) return 0;
 else if(phi < sectors) return theta*sectors + phi;
 
 else return theta*sectors;
}


int MSweepRotateCreator::texVertexAt(int phi, int theta, int sectors, int rings)
{
 return theta*(sectors + 1) + phi;
}


MSweepRotateCreator::MSweepRotateCreator(void) : MCreator()
{

}


MShape *MSweepRotateCreator::create(MModel *parent)
{
 int i, j;

 int rings, sectors;
 float sizeU, sizeV;
 float tx, ty;
 MVertex *v;
 QValueVector<MVector> vertices;
 MVector x, x_prime;
 MVector axis;
 MMatrix R;
 float angle;
 
 MShape *shape;
 
 sectors = _parameters[QString("Sectors")].toInt();
 
 sizeU = _parameters[QString("U-Size")].toDouble();
 sizeV = _parameters[QString("V-Size")].toDouble();
 
 shape = MDocument::get()->getSelectedShape();
 if(shape)
 {
  shape->setName(shape->getName() + " sweep rotate");

  // vertices
  QPtrList<MVertex> &vl = shape->getVertexList();
  for(v = vl.first(); v; v = vl.next())
  {
   vertices.push_back(v->getPosition());
  }
  rings = (int)vertices.size();
  vl.clear();

  axis = shape->getUp();
  for(j = 0; j < rings; j++)
  {
   for(i = 0; i < sectors; i++)
   {
    angle = 360.0*((float)i)/((float)sectors);
    R.loadRotation(angle, axis);
   
    x = vertices[j];
    x_prime = R*x;
      
    shape->addVertex(new MVertex(x_prime[0], x_prime[1], x_prime[2]));
   }
  }
 
   // textureVertices
   
  for(j = 0; j < rings; j++)
  {
   for(i = 0; i <= sectors; i++)
   {
    tx = sizeU*((float)i)/((float)sectors);
    ty = sizeV*((float)j)/((float)(rings - 1));

    shape->addTextureVertex(new MVector(tx, ty, 0.0));
   }
  }
  
  // facets
  for(j = 0; j < rings-1; j++)
  {
   for(i = 0; i < sectors; i++)
   {
    shape->addFacet(new MFacet(vertexAt(i+1, j, sectors, rings),
                               vertexAt(i, j+1, sectors, rings),
                               vertexAt(i, j, sectors, rings),
			      
	 		       texVertexAt(i+1, j, sectors, rings),
                               texVertexAt(i, j+1, sectors, rings),
                               texVertexAt(i, j, sectors, rings)));
			      
			      
			      
    shape->addFacet(new MFacet(vertexAt(i+1, j, sectors, rings),
                               vertexAt(i+1, j+1, sectors, rings),
                               vertexAt(i, j+1, sectors, rings),
		 	      
	 		       texVertexAt(i+1, j, sectors, rings),
                               texVertexAt(i+1, j+1, sectors, rings),
                               texVertexAt(i, j+1, sectors, rings)));
   }
  }
 }
 
 return shape;
}
