
/*
 This file is part of moth. moth is a program for creating
 and editing textured 3D models.
 
 Copyright (C) 2004 Peter Uray.

 moth is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 moth is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with moth; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <iostream>

#include <unistd.h>

#include <qapplication.h>
#include <qpushbutton.h>

#include <qwindowsstyle.h>
#include <qlabel.h>
#include <qfont.h>
#include <qnamespace.h>
#include <qregexp.h>

#include <MPreferences.h>
#include <MSplashScreen.h>
#include <MCoordinateSystem.h>
#include <MModelView.h>
#include <MStructureView.h>
#include <MMainWindow.h>
#include <MDocument.h>

int main(int argc, char ** argv)
{
 QString mypath;
 QFont f;
 QApplication a(argc, argv);
 
 /// Changedir to the modeler path.
 mypath = getenv("_");
 mypath = mypath.replace(QRegExp("\\"), "/");
 mypath.remove(mypath.findRev('/'), mypath.length());
 chdir(mypath.utf8());

 a.setStyle(new QWindowsStyle());

 // Replace by a preference.
 a.setFont(QFont("clean", 8), TRUE);
 
 // This call causes the preferences to be constructed and read before use.
 MPreferences::initialize();
 MPreferences::get()->readPreferences();

 MMaterialManager::get()->setProjectDirectory(MPreferences::get()->getProjectLocation());
 
 MSplashScreen::expose();
 MSplashScreen::display("Version 1 / Release 1");
 sleep(2);
 MSplashScreen::conceal();

 MMainWindow::get()->resize(QApplication::desktop()->width(), QApplication::desktop()->height());
 a.setMainWidget(MMainWindow::get());  
 MMainWindow::get()->show();

 a.connect(&a, SIGNAL(lastWindowClosed()), &a, SLOT(quit()));

 if(argc > 1)
 {
  MDocument::get()->loadScene(QString(argv[1]));
  MStructureView::get()->update();
 }
 
 return a.exec();

}

